<?php
require_once(dirname(__FILE__).'/constants.php');
require_once(dirname(__FILE__).'/constants_changelog.php');
require_once(dirname(__FILE__).'/session.php');
require_once 'MDB2.php';
PEAR::setErrorHandling(PEAR_ERROR_RETURN);

/**
 * getPEARErrorInfo - Returns full info from PEAR::Error
 */
if (!function_exists('getPEARErrorInfo')) {
	function getPEARErrorInfo($dberrobj, $sep = "\n") {
		if (PEAR::isError($dberrobj)) {
			$dbmsg = $dberrobj->getUserInfo();
			$dbmsgd = $dberrobj->getDebugInfo();
			$dbmsgd = (strcmp($dbmsg, $dbmsgd) == 0) ? '' : $sep . "dbg: " . $dbmsgd;
			return "msg: " . $dberrobj->getMessage() . "(" . $dberrobj->getCode() . ")" . $sep
			       . "dbgmsg: " . $dbmsg . $dbmsgd;
		} else {
			return "";
		}
	}
}

class IBDB {
	public $connection;
	public $db_ok;

	function IBDB() {
		global $logger;

		$dsn = DB_ENGINE.'://'.DB_USER.':'.DB_PASS.'@'.DB_HOST.'/'.DB_FILE;
		$this->connection = MDB2::factory($dsn, false);
		if (PEAR::isError($this->connection)) {
			$this->db_ok = false;
			$logger->log('IBDB Fatal Error: '.getPEARErrorInfo($this->connection, "\n"));
			//header("Location: closed.php");
			//die();
			return;
		}
		$this->db_ok = true;

		$this->connection->setOption('field_case', CASE_LOWER);
		$this->connection->setOption('portability', MDB2_PORTABILITY_NUMROWS | MDB2_PORTABILITY_FIX_CASE);
		$this->connection->setFetchMode(MDB2_FETCHMODE_ASSOC);
		//$this->connection->setFetchMode(MDB2_FETCHMODE_OBJECT);
		$this->connection->LoadModule('Extended');

//		$logger->log("ibase.timestampformat BEFORE=".ini_get("ibase.timestampformat"));
//		if (function_exists('ibase_timefmt')) {
//			ibase_timefmt("%d.%m.%Y", IBASE_TIMESTAMP);
//			ibase_timefmt("%d.%m.%Y", IBASE_DATE);
//		} else {
//			ini_set("ibase.timestampformat", "%d.%m.%Y");
//			//ini_set("ibase.timeformat", "%H:%M:%S");
//			ini_set("ibase.dateformat", "%d.%m.%Y");
//		}
//		$logger->log("ibase.timestampformat AFTER=".ini_get("ibase.timestampformat"));
	}

	function transactionStart() {
		return true;

		/*global $logger;

		$result = false;
		if (true/*$this->connection->supports('transactions')* /) {
				$result = $this->connection->beginTransaction();
		}
		$logger->log("transactionStart=\n".print_r($result, true));
		return $result;*/
	}

	function transactionCommit() {
		return true;

		/*global $logger;

		$result = false;
		if (true/*$this->connection->inTransaction()* /) {
			$result = $this->connection->commit();
		}
		$logger->log("transactionCommit=\n".print_r($result, true));
		return $result;*/
	}

	function transactionRollback() {
		return true;

		/*global $logger;

		$result = false;
		if (true/*$this->connection->inTransaction()* /) {
			$result = $this->connection->rollback();
		}
		$logger->log("transactionRollback=\n".print_r($result, true));
		return $result;*/
	}

	/**
	 * query - Performs the given query on the database and
	 * returns the result, which may be false, true or a
	 * resource identifier.
	 */
	function query($query) {
		return $this->connection->query($query);
	}

	/**
	 * prepare - wrapper
	 *
	 * @param      $query
	 * @param      $types
	 *
	 * @param null $result_types
	 *
	 * @return object
	 */
	function prepare($query, $types = null, $result_types = null) {
		global $logger;

		$result = $this->connection->prepare($query, $types, $result_types);
		if (PEAR::isError($result)) {
			$logger->log("prepare($query, $types) ERR:\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			return null;
		}
		return $result;
	}

	/**
	 * execute - wrapper
	 *
	 * @param $preparedquery
	 * @param $data
	 *
	 * @return null
	 */
	function execute($preparedquery, $data) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		if (!$preparedquery) {
			$logger->log("execute ERR0 (called for wrong obj):\n" . print_r($preparedquery, true), PEAR_LOG_ERR);
			return null;
		}

		//$logger->log("execute Q:\n" . print_r($preparedquery, true));
		$result = $preparedquery->execute($data);

		if (PEAR::isError($result)) {
			$logger->log("execute ERR:\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			return null;
		}
		//$logger->log("execute OK:\n" . print_r($result, true));
		if (!isset($preparedquery->is_manip) || !$preparedquery->is_manip) {
			$res = $result->fetchAll();
			$result->free();
		} else
			$res = $result;
		return $res;
	}

	/**
	 * confirmUserPass - Checks whether or not the given
	 * username is in the database, if so it checks if the
	 * given password is the same password in the database
	 * for that user. If the user doesn't exist or if the
	 * passwords don't match up, it returns null.
	 * On success it returns user_id
	 * @param $ofnumber
	 * @param $username
	 * @param null $password
	 * @return null|int
	 */
	function confirmUserPass($ofnumber, $username, $password = null) {
		global $logger;

		if (!get_magic_quotes_gpc())
			$username = addslashes($username);

		$num = strlen($ofnumber) > 0 ? "'" . substr($ofnumber, 0, MAX_LEN_OFNUMBER) . "'" : 'null';
		$psw = $password == null ? 'null' : "'" . substr($password, 0, MAX_LEN_OFPASSWORD) . "'";
		$pswl = $password == null ? '' : ', ***';

		$q = "select * from of_web_authorize("
			. "$num,"
			. "'" . substr($username, 0, MAX_LEN_OFUSERNAME) . "',"
			. $psw
			. ")";
		$result = $this->connection->query($q);
		if (PEAR::isError($result)) {
			$logger->log("confirmUserPass($num, '$username'$pswl):". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return null;
		}
		if (!($dbarray = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) || !is_numeric($dbarray['user_id'])) {
			$logger->log("confirmUserPass($num, '$username'$pswl):".print_r($dbarray, true), PEAR_LOG_ERR);
			return null;
		}
		//$logger->log("confirmUserPass($num, '$username'$pswl):\n".print_r($dbarray, true));
		return $dbarray['user_id'];
	}

	/**
	 * confirmUserID - Checks whether or not the given
	 * username is in the database, if so it checks if the
	 * given userwebid is the same userwebid in the database
	 * for that user. If the user doesn't exist or if the
	 * userwebids don't match up, it returns an error code
	 * (1 or 2). On success it returns 0.
	 */
	function confirmUserID($ofnumber, $userwebid){
		global $logger;

		$logger->log("confirmUserID('$ofnumber', $userwebid)", PEAR_LOG_DEBUG);

		/* Verify that user is in database */
		$q = "select userwebid from ".TB_WEB_CACHE." where of_id in"
			." (select of_id from tb_offices where of_number='$ofnumber')";
		$result = $this->connection->query($q);
		if (PEAR::isError($result)) {
			$logger->log(getPEARErrorInfo($result), PEAR_LOG_ERR);
			return 1;
		}
		if (!$dbarray = $result->fetchRow()) {
			$logger->log("confirmUserID: NULL:".print_r($dbarray, true), PEAR_LOG_DEBUG);
			return 1; //Indicates username failure
		}

		/* Retrieve userwebid from result, strip slashes */
		$dbarray['userwebid'] = stripslashes($dbarray['userwebid']);
		$userwebid = stripslashes($userwebid);

		/* Validate that userwebid is correct */
		if ($userwebid == $dbarray['userwebid'])
			return 0; //Success! Username and userwebid confirmed
		else
			return 2; //Indicates userwebid invalid
	}

	/**
	 * registerLoginSession - Creates record in the database's
	 * main web_cache table.
	 * Returns true on success, false otherwise.
	 */
	function registerLoginSession($userid, $ofid, $userwebid, $time) {
		global $logger;

		$ofid = strlen($ofid) > 0 ? "'".$ofid."'" : 'null';
		$q = "select * from of_web_registerloginsession($userid, $ofid, '$userwebid', $time)";
		$result = $this->connection->query($q);

		if (PEAR::isError($result)) {
			$logger->log("$q CHECK:". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		}
		if (($dbarray = $result->fetchRow()))
			return $dbarray['op_result'];

		return false;
	}

	/**
	 * getQueryResultAsArray
	 * @param $query
	 * @param bool $singleton
	 * @param null $types
	 * @return array|null
	 */
	function getQueryResultAsArray($query, $singleton = false, $types = null) {
		global $logger;

		$result = $this->connection->queryAll($query, $types);
		if (PEAR::isError($result)) {
			$logger->log("getQueryResultAsArray($query) ERR:" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
		}
		$logger->log("getQueryResultAsArray($query) RES:" . print_r($result, true), PEAR_LOG_DEBUG);

		if ($singleton === false)
			return $result;
		else {
			if (empty($result))
				return null;
			else
				return $result[0];
		}
	}

	/**
	 * getExtNumbersListByOfficeID
	 */
	function getExtNumbersListByOfficeID($ofid){
		return $this->getQueryResultAsArray("select * from of_get_numbers($ofid)");
	}

	/**
	 * getExtNumbersList_htmlcode_ex
	 */
	function getExtNumbersList_htmlcode_ex($arr){
		$result = "";//"var int_nums = new Array();\n";
		foreach ($arr as $row) {
			$result .= "numberlist['".addslashes(htmlspecialchars($row['external_number']))."'] = ".$row['number_id'].";";
		}
		return $result;
	}

	/**
	 * getIntNumbersListByOfficeID
	 */
	function getIntNumbersListByOfficeID($ofid){
		return $this->getQueryResultAsArray("select * from of_get_int_numbers($ofid)");
	}

	/**
	 * getIntNumbersList_htmlcode
	 */
	function getIntNumbersList_htmlcode($arr, $selected_num_id){
		$result = "";
		foreach ($arr as $row) {
			$result .= /*\t*/"<option value=\"".$row['internal_number_id']."\"";
			if ($selected_num_id == $row['internal_number_id']) {
				$result .= " selected";
			}
			$result .= ">".addslashes(htmlspecialchars($row['internal_number']))."</option>";
		}
		//$result .= print_r($arr, true);
		return $result;
	}

	/**
	 * getIntNumbersList_htmlcode_ex
	 */
	function getIntNumbersList_htmlcode_ex($arr){
		$result = "";//"var int_nums = new Array();\n";
		foreach ($arr as $row) {
			$result .= "int_nums['".addslashes(htmlspecialchars($row['internal_number']))."'] = ".$row['internal_number_id'].";";
		}
		return $result;
	}

	/**
	 * getDepartmentsByOfficeID
	 */
	function getDepartmentsByOfficeID($ofid){
		return $this->getQueryResultAsArray("select * from of_get_departments($ofid)");
	}

	/**
	 * getDialListsByOfficeID
	 */
	function getDialListsByOfficeID($ofid){
		return $this->getQueryResultAsArray("select * from of_get_diallists($ofid)");
	}

	/**
	 * getDialLists_htmlcode_ex
	 */
	function getDialLists_htmlcode_ex($arr){
		$result = "";
		foreach ($arr as $row) {
			$result .= "diallists['".addslashes(htmlspecialchars($row['dials_name']))."'] = ".$row['dials_id'].";";
		}
		return $result;
	}

	/**
	 * getGraphListsByOfficeID
	 */
	function getGraphListsByOfficeID($ofid){
		return $this->getQueryResultAsArray("select * from of_get_graphlists($ofid)");
	}

	/**
	 * getGraphLists_htmlcode_ex
	 */
	function getGraphLists_htmlcode_ex($arr){
		$result = "";
		foreach ($arr as $row) {
			$result .= "graphlists['".addslashes(htmlspecialchars($row['graph_name']))."'] = ".$row['graph_id'].";";
		}
		return $result;
	}

	/**
	 * getDayGroupsByOfficeID
	 */
	function getDayGroupsByOfficeID($ofid){
		return $this->getQueryResultAsArray("select * from of_get_daygroups($ofid)");
	}

	/**
	 * getDayGroups_htmlcode_ex
	 */
	function getDayGroups_htmlcode_ex($arr){
		$result = "";
		foreach ($arr as $row) {
			$result .= "daygrouplist['".addslashes(htmlspecialchars($row['daygroup_name']))."'] = ".$row['daygroup_id'].";";
		}
		return $result;
	}

	/**
	 * getWeekDaysMaskByDayGroupID
	 */
	function getWeekDaysMaskByDayGroupID($dgid){
		return $this->getQueryResultAsArray("select * from of_get_weekdays_mask($dgid)");
	}

	/**
	 * updateNumberBW
	 */
	function updateNumberBW($ofid, $bwnumber, $exactmatch, $listname) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		$pq = $this->prepare("insert into tb_numbers_$listname(of_id, caller_id, exactmatch) values ($ofid, ?, $exactmatch)",
								array('text'));
		$result = $this->execute($pq, array($bwnumber));

		if (PEAR::isError($result))
			$logger->log("updateNumberBW($ofid, '$bwnumber', $exactmatch, '$listname')):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
		$result = intval(PEAR::isError($result));
		$logger->log("updateNumberBW($ofid, '$bwnumber', $exactmatch, '$listname') result:".print_r($result, true), PEAR_LOG_DEBUG);

		return $result;
	}

	/**
	 * updateNumberExt
	 */
	function updateNumberExt($ofid, $ext_num_id, $ext_number, $about) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		$about = str_ireplace('\'', '"', $about);
		$params = "$ofid, $ext_num_id, '$ext_number', '$about'";
		$q = "select * from of_update_ext_number($params)";
		$result = $this->connection->queryAll($q);
		if (PEAR::isError($result)) {
			$logger->log("updateNumberExt($params) ERR:". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
		}
		$logger->log("updateNumberExt($params) result:".print_r($result, true));

		//changelog
		$this->logOfficeChange($ext_num_id == -1 ? CL_OF_EXTNUMBER_NEW :
							($ext_number == '' ? CL_OF_EXTNUMBER_DELETE : CL_OF_EXTNUMBER_CHANGE),
			$ofid, 0,
			($ext_num_id == -1 ? 'Добавление' : ($ext_number == '' ? 'Удаление' : 'Изменение')) .
				(" внешнего номера '$ext_number': ") .
				($result == null ? 'ошибка.' : 'успешно.')
			);

		return $result;
	}

	/**
	 * updateNumberInt
	 */
	function updateNumberInt($ofid, $internal_number_id, $internal_number, $dials, $ignore_graph, $about) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		if (!is_numeric($dials))
			$dials = 'null';
		$about = str_ireplace('\'', '"', $about);

		$q = "select * from of_update_int_number($ofid, $internal_number_id, '$internal_number', $dials, $ignore_graph, '$about')";
		$result = $this->connection->queryAll($q);
		if (PEAR::isError($result)) {
			$logger->log("updateNumberInt($ofid, $internal_number_id, '$internal_number', $dials, $ignore_graph, '$about') ERR:". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
		}
		//$logger->log("updateNumberInt($ofid, $internal_number_id, '$internal_number', $dials, $ignore_graph, '$about') result:".print_r($result, true));

		//changelog
		$this->logOfficeChange($internal_number_id == -1 ? CL_OF_INTNUMBER_NEW :
							($internal_number == '' ? CL_OF_INTNUMBER_DELETE : CL_OF_INTNUMBER_CHANGE),
			$ofid, 0,
			($internal_number_id == -1 ? 'Добавление' : ($internal_number == '' ? 'Удаление' : 'Изменение')) .
				(" внутреннего номера '$internal_number': ") .
				($result == null ? 'ошибка.' : 'успешно.')
			);

		return $result;
	}

	/**
	 * updateDepartment
	 */
	function updateDepartment($ofid, $depid, $dep_name, $dials_id) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

//      $q = "SELECT * FROM OF_UPDATE_DEPARTMENT($ofid, $depid, '$dep_name', $dials_id)";
//      $result = $this->connection->query($q);
//    if (PEAR::isError($result)) {
//      $logger->log("updateDepartment($ofid, $depid, '$dep_name', $dials_id) ERR:".getPEARErrorInfo($result));
//      return null;
//    }
//    $temp = null;
//    $dbarray = array();
//    while ($result->fetchInto($temp)) {
//      $dbarray[] = $temp;
//    }
//    $logger->log("updateDepartment($ofid, $depid, '$dep_name', $dials_id).result=".print_r($dbarray, true));
//      return $dbarray;

		$q = "select * from of_update_department($ofid, $depid, '$dep_name', $dials_id)";
		$result = $this->connection->queryAll($q);
		if (PEAR::isError($result)) {
			$logger->log("updateDepartment($ofid, $depid, '$dep_name', $dials_id).ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
		}
		$logger->log("updateDepartment($ofid, $depid, '$dep_name', $dials_id).result=".print_r($result, true), PEAR_LOG_DEBUG);
		return $result;
	}

	/**
	 * updateDialList
	 */
	function updateDialList_OLD($ofid, $dials_id, $dials_name,
							$dial_mode,
							$wait_answer_pause,	$redial_pause, $redial_count,
							$wait_answer_phrase, $wait_redial_phrase,
							$say_aon, $translate_real_aon, $do_recording, $call_is_owner,
							$can_redirect, $can_conference, $wait_conn_phrase,
							$new_dial_symbols, $dial_ret_symbols, $conn_ret_symbols, $ivr_ret_symbols,
							$ready_2_dial_phrase) {
		global $session, $logger;

		if ($session->isDemo() || $dials_name === null)
			return null;

		if ($redial_count <= 0)
			$redial_count = 1;
		if ($session->isOwner() && $dials_id == -1) {
			$new_dial_symbols = '##';
			$dial_ret_symbols = '*';
			$conn_ret_symbols = '#';
			$ivr_ret_symbols = '0';
		}

		/*$dials_name = str_ireplace('\'', '"', $dials_name);
		$wait_answer_phrase = str_ireplace('\'', '"', $wait_answer_phrase);
		$wait_redial_phrase = str_ireplace('\'', '"', $wait_redial_phrase);
		$wait_conn_phrase = str_ireplace('\'', '"', $wait_conn_phrase);*/

		$params = "$ofid, $dials_id, '$dials_name', " .
			"$dial_mode, " .
			"$wait_answer_pause, $redial_pause, $redial_count, " .
			"'$wait_answer_phrase', '$wait_redial_phrase', " .
			intval($say_aon).", ".intval($translate_real_aon).", ".
			intval($do_recording).", ".intval($call_is_owner).", ".intval($can_redirect).", ".
			intval($can_conference).", '$wait_conn_phrase', ".
			"'$new_dial_symbols', '$dial_ret_symbols', '$conn_ret_symbols', '$ivr_ret_symbols', ".
			"'$ready_2_dial_phrase'";
		/*$q = "select * from of_update_diallist($params)";
		$result = $this->connection->queryAll($q);
		if (PEAR::isError($result)) {
			$logger->log("updateDialList($params).ERR=".getPEARErrorInfo($result));
			$result = null;
		}
		$logger->log("updateDialList($params).result=".print_r($result, true));
		return $result;*/

		if ($session->isOwner() && $dials_id >= 0) {
			$pq = $this->prepare("select * from of_update_diallist(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
				array('integer', 'integer', 'text',
					'integer',
					'integer', 'integer', 'integer', 'text', 'text',
					'integer', 'integer', 'integer', 'integer',
					'integer', 'integer', 'text',
					'text', 'text', 'text', 'text',
					'text'));
			$result = $this->execute($pq, array(
				$ofid, $dials_id, $dials_name,
				null/*$dial_mode*/,
				$wait_answer_pause, $redial_pause, $redial_count, $wait_answer_phrase, $wait_redial_phrase,
				null/*intval($say_aon)*/, intval($translate_real_aon), intval($do_recording), null/*intval($call_is_owner)*/,
				null/*intval($can_redirect)*/, null/*intval($can_conference)*/, $wait_conn_phrase,
				null/*$new_dial_symbols*/, null/*$dial_ret_symbols*/, null/*$conn_ret_symbols*/, null/*$ivr_ret_symbols*/,
				null/*$ready_2_dial_phrase*/
			));
		} else {
			$pq = $this->prepare("select * from of_update_diallist(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
				array('integer', 'integer', 'text',
					'integer',
					'integer', 'integer', 'integer', 'text', 'text',
					'integer', 'integer', 'integer', 'integer',
					'integer', 'integer', 'text',
					'text', 'text', 'text', 'text',
					'text'));
			$result = $this->execute($pq, array(
				$ofid, $dials_id, $dials_name,
				$dial_mode,
				$wait_answer_pause, $redial_pause, $redial_count, $wait_answer_phrase, $wait_redial_phrase,
				intval($say_aon), intval($translate_real_aon), intval($do_recording), intval($call_is_owner),
				intval($can_redirect), intval($can_conference), $wait_conn_phrase,
				$new_dial_symbols, $dial_ret_symbols, $conn_ret_symbols, $ivr_ret_symbols,
				$ready_2_dial_phrase
			));
		}

		if (PEAR::isError($result))
			$logger->log("updateDialList($params):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
		/*else
			$logger->log("updateDialList($params).result=".print_r($result, true), PEAR_LOG_DEBUG);*/

		$result = (empty($result) || PEAR::isError($result)) ? null : $result;

		//changelog
		$this->logOfficeChange($dials_id < 0 ? CL_OF_DIALS_NEW :
				($dials_name == '' ? CL_OF_DIALS_DELETE : CL_OF_DIALS_CHANGE),
			$ofid, 0,
			($dials_id == -1 ? 'Добавление' : ($dials_name == '' ? 'Удаление' : 'Изменение')) .
				(" списка дозвона '$dials_name': ") .
				($result == null ? 'ошибка.' : 'успешно.')
		);

		return $result;
	}

	function updateDialList($ofid, $dials_id, $dials_name,
							$dial_mode,
							$wait_answer_pause,	$redial_pause, $redial_count,
							$wait_answer_phrase, $wait_redial_phrase,
							$say_aon, $translate_real_aon, $do_recording, $call_is_owner,
							$can_redirect, $can_conference, $wait_conn_phrase,
							$new_dial_symbols, $dial_ret_symbols, $conn_ret_symbols, $ivr_ret_symbols,
							$ready_2_dial_phrase,$dtmf_retranslate) {
		global $session, $logger;

		if ($session->isDemo() || $dials_name === null)
			return null;

		if ($redial_count <= 0) $redial_count = 1;
		
		$new_dial_symbols = NEWDIALLIST_NEW_DIAL_SYMBOLS;
		$dial_ret_symbols = NEWDIALLIST_DIAL_RET_SYMBOLS;
		$conn_ret_symbols = NEWDIALLIST_CONN_RET_SYMBOLS;
		$ivr_ret_symbols = NEWDIALLIST_IVR_RET_SYMBOLS;

		/*$dials_name = str_ireplace('\'', '"', $dials_name);
		$wait_answer_phrase = str_ireplace('\'', '"', $wait_answer_phrase);
		$wait_redial_phrase = str_ireplace('\'', '"', $wait_redial_phrase);
		$wait_conn_phrase = str_ireplace('\'', '"', $wait_conn_phrase);*/

		$params = "$ofid, $dials_id, '$dials_name', " .
			"$dial_mode, " .
			"$wait_answer_pause, $redial_pause, $redial_count, " .
			"'$wait_answer_phrase', '$wait_redial_phrase', " .
			intval($say_aon).", ".intval($translate_real_aon).", ".
			intval($do_recording).", ".intval($call_is_owner).", ".intval($can_redirect).", ".
			intval($can_conference).", '$wait_conn_phrase', ".
			"'$new_dial_symbols', '$dial_ret_symbols', '$conn_ret_symbols', '$ivr_ret_symbols', ".
			"'$ready_2_dial_phrase', ".intval($dtmf_retranslate);

			
			$pq = $this->prepare("select * from of_update_diallist(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
				array('integer', 'integer', 'text',
					'integer',
					'integer', 'integer', 'integer', 'text', 'text',
					'integer', 'integer', 'integer', 'integer',
					'integer', 'integer', 'text',
					'text', 'text', 'text', 'text',
					'text', 'integer'));
			$result = $this->execute($pq, array(
				$ofid, $dials_id, $dials_name,
				$dial_mode,
				$wait_answer_pause, $redial_pause, $redial_count, $wait_answer_phrase, $wait_redial_phrase,
				intval($say_aon), intval($translate_real_aon), intval($do_recording), intval($call_is_owner),
				intval($can_redirect), intval($can_conference), $wait_conn_phrase,
				$new_dial_symbols, $dial_ret_symbols, $conn_ret_symbols, $ivr_ret_symbols,
				$ready_2_dial_phrase, intval($dtmf_retranslate)
			));
			

		if (PEAR::isError($result))
			$logger->log("updateDialList($params):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
		/*else
			$logger->log("updateDialList($params).result=".print_r($result, true), PEAR_LOG_DEBUG);*/

		$result = (empty($result) || PEAR::isError($result)) ? null : $result;

		//changelog
		$this->logOfficeChange($dials_id < 0 ? CL_OF_DIALS_NEW :
				($dials_name == '' ? CL_OF_DIALS_DELETE : CL_OF_DIALS_CHANGE),
			$ofid, 0,
			($dials_id == -1 ? 'Добавление' : ($dials_name == '' ? 'Удаление' : 'Изменение')) .
				(" списка дозвона '$dials_name': ") .
				($result == null ? 'ошибка.' : 'успешно.')
		);

		return $result;
	}
	
	
	
	/**
	* updateDialsMultiple
	*/
	function updateDialsMultiple($ofid, $data) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		//$logger->log("updateDialsMultiple($ofid, $data).DATA=\n".print_r($data, true), PEAR_LOG_DEBUG);

		$q = 'insert into tb_dials values (?, ?, ?, ?, ?)';
		$result = $this->connection->prepare($q);
		if (PEAR::isError($result)) {
			$logger->log("updateDialsMultiple($ofid, $data).PREPARE.ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
			return $result;
		}

		$this->connection->extended->executeMultiple($result, $data);
		if (PEAR::isError($result)) {
			$logger->log("updateDialsMultiple($ofid, $data).EXEC.ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
			return $result;
		}
		$result->free();

		$result = true;
		//$logger->log("updateDialsMultiple($ofid, $data).result=\n".print_r($result, true), PEAR_LOG_DEBUG);
		return $result;
	}

	/**
	 * updateGraphList
	 */
	function updateGraphList($ofid, $graph_id, $graph_name) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		$graph_name = str_ireplace('\'', '"', $graph_name);

		$q = "select * from of_update_graphlist($ofid, $graph_id, '$graph_name')";
		$result = $this->connection->queryAll($q);
		if (PEAR::isError($result)) {
			$logger->log("updateGraphList($ofid, $graph_id, '$graph_name').ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
		}
		$logger->log("updateGraphList($ofid, $graph_id, '$graph_name').result=".print_r($result, true), PEAR_LOG_DEBUG);
		return $result;
	}

	/**
	 * updateGraphsMultiple
	 * @param $ofid
	 * @param $data
	 * @return bool|null
	 */
	function updateGraphsMultiple($ofid, $data) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		$logger->log("updateGraphsMultiple($ofid, $data).DATA=\n".print_r($data, true));

		$q = 'insert into tb_graphics values (?, ?, ?, ?)';
		$result = $this->connection->prepare($q);
		if (PEAR::isError($result)) {
			$logger->log("updateGraphsMultiple($ofid, $data).PREPARE.ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
			return $result;
		}

		//DB:  $result = $this->connection->executeMultiple($result, $data);
		$this->connection->extended->executeMultiple($result, $data);
		if (PEAR::isError($result)) {
			$logger->log("updateGraphsMultiple($ofid, $data).EXEC.ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
			return $result;
		}
		$result->free();

		$result = true;
		$logger->log("updateGraphsMultiple($ofid, $data).result=".print_r($result, true), PEAR_LOG_DEBUG);
		return $result;
	}

	/**
	 * updateWeekDays_Mask
	 */
	function updateWeekDays_Mask($daygroup_id, $weekdays_mask) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		$q = "select * from of_set_weekdays_mask($daygroup_id, $weekdays_mask)";
		$result = $this->connection->queryAll($q);
		if (PEAR::isError($result)) {
			$logger->log("updateWeekDays_Mask($daygroup_id, $weekdays_mask).ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
		}
		$logger->log("updateWeekDays_Mask($daygroup_id, $weekdays_mask).result=".print_r($result, true), PEAR_LOG_DEBUG);
		return $result;
	}

	/**
	 * updateDayGroup
	 */
	function updateDayGroup($ofid, $daygroup_id, $daygroup_name) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		$q = "select * from of_update_daygroup($ofid, $daygroup_id, '$daygroup_name')";
		$result = $this->connection->queryAll($q);
		if (PEAR::isError($result)) {
			$logger->log("updateDayGroup($ofid, $daygroup_id, '$daygroup_name').ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
		}
		$logger->log("updateDayGroup($ofid, $daygroup_id, '$daygroup_name').result=".print_r($result, true), PEAR_LOG_DEBUG);

		//changelog
		$this->logOfficeChange($daygroup_id == -1 ? CL_OF_DAYGROUP_NEW :
							($daygroup_name == '' ? CL_OF_DAYGROUP_DELETE : CL_OF_DAYGROUP_CHANGE),
			$ofid, 0,
			($daygroup_id == -1 ? 'Добавление' : ($daygroup_name == '' ? 'Удаление' : 'Изменение')) .
				(" группы дней '$daygroup_name': ") .
				($result == null ? 'ошибка.' : 'успешно.')
			);

		return $result;
	}

	/**
	 * updateDaysMultiple
	 */
	function updateDaysMultiple($ofid, $data) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		$logger->log("updateDaysMultiple($ofid, $data).DATA=\n".print_r($data, true));

		$q = 'insert into tb_hollydays values (?, ?, ?)';
		$result = $this->connection->prepare($q);
		if (PEAR::isError($result)) {
			$logger->log("updateDaysMultiple($ofid, $data).PREPARE.ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
			return $result;
		}

		//DB: $result = $this->connection->executeMultiple($result, $data);
		$this->connection->extended->executeMultiple($result, $data);
		if (PEAR::isError($result)) {
			$logger->log("updateDaysMultiple($ofid, $data).EXEC.ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
			return $result;
		}
		$result->free();

		$result = true;
		$logger->log("updateDaysMultiple($ofid, $data).result=\n".print_r($result, true), PEAR_LOG_DEBUG);
		return $result;
	}

	/**
	 * getOfficeUserInfo - Returns the result array from a
	 * query asking for all information stored regarding
	 * the given username. If query fails, NULL is returned.
	 */
	function getOfficeUserInfo($ofnumber, $username){
		global $logger;

		$num = $ofnumber ? "'" . substr($ofnumber, 0, MAX_LEN_OFNUMBER) . "'" : 'null';
		$q = "select * from of_web_authorize("
			. "$num,"
			. "'" . substr($username, 0, MAX_LEN_OFUSERNAME) . "',"
			. "null"
			. ")";
		$result = $this->connection->query($q);
		if (PEAR::isError($result)) {
			$logger->log("getOfficeUserInfo($num, '$username'):". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return null;
		}

		/* Error occurred, return given name by default */
		if(!($dbarray = $result->fetchRow()) || $ofnumber && !$dbarray['of_id']) {
			return null;
		}

		/* Return result array */
		//$logger->log("getOfficeUserInfo($num, '$username'):".print_r($dbarray, true), PEAR_LOG_DEBUG);
		return $dbarray;
	}

	/**
	 * getOfficeParams - Returns the result array from a
	 * query asking for all custom office parameters.
	 * If query fails, NULL is returned.
	 */
	function getOfficeParams($ofid) {
		//global $logger;

		$res = $this->getQueryResultAsArray("select * from of_get_params($ofid)");
		if ($res) {
			$keyedres = array();
			foreach ($res as $prm) {
				$keyedres[$prm['pname']] = array('pvalue' => $prm['pvalue'], 'ptype' => $prm['ptype']);
			}
			//$logger->log("getOfficeParams($ofid):\n".print_r($keyedres, true), PEAR_LOG_DEBUG);
			return $keyedres;
		}
		return null;
	}

	/**
	 * setOfficeParam - Sets custom office parameter.
	 * Returns db code.
	 */
	function setOfficeParam($ofid, $pname, $pvalue) {
		global $session, $logger;

		$logger->log("setOfficeParam($ofid, '$pname', '$pvalue')");

		if ($session->isDemo())
			return true;

		$q = "select * from of_set_param($ofid, '$pname', '$pvalue')";
		$result = $this->connection->query($q);
		if (PEAR::isError($result)) {
			$logger->log("setOfficeParam($ofid, '$pname', '$pvalue'):". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return null;
		}

		/* Error occurred, return null by default */
		if((!$dbarray = $result->fetchRow()) || !$dbarray['res']) {
			return null;
		}
		/* Return result array */
		return $dbarray['res'];
	}

	/**
	 * setOfficeMenu - Associates office with specified dynamic menu.
	 * Returns true on successful update, false on failure.
	 * @param $ofid
	 * @param $menuid
	 * @param string $menuname
	 * @return bool
	 */
	function setOfficeMenu($ofid, $menuid, $menuname = '') {
		global $session, $logger;

		$logger->log("setOfficeMenu($ofid, $menuid, '$menuname')");

		if ($session->isDemo())
			return true;

		$q = "select * from of_set_menu($ofid, $menuid)";
		$result = $this->connection->query($q);
		if (PEAR::isError($result)) {
			$logger->log("setOfficeMenu($ofid, $menuid, '$menuname'):". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		}

		/* Error occurred, return false by default */
		if((!$dbarray = $result->fetchRow()) || !isset($dbarray['res'])) {
			$logger->log("setOfficeMenu($ofid, $menuid, '$menuname') -> invalid result:".print_r($dbarray, true), PEAR_LOG_ERR);
			return false;
		}

		$result = $dbarray['res'] == 0;

		//changelog
		if ($result)
			$this->logOfficeChange(CL_OF_MENU_ASSIGN, $ofid, 0, "Сценарий работы изменен на #$menuid, '$menuname'.");
		return $result;
	}

	/**
	 * logOfficeChange - Logs change event to database.
	 * Returns false or code from database :)
	 * @param $log_changetype
	 * @param $log_office_id
	 * @param $log_userdata
	 * @param $log_msg
	 * @return bool
	 */
	function logOfficeChange($log_changetype, $log_office_id, $log_userdata, $log_msg) {
		global $session, $logger;

		//changelog
		$log_office_id_in = $log_office_id ? $log_office_id : 'null';
		$log_office_user_id = $session->logged_in ? $session->userid : -1;
//		$log_msg .= $session->logged_in ? " Пользователь '{$session->username}' (id:{$session->userid})." : "";
//		$log_msg .= $session->logged_in ? " Пользователь '{$session->username}' (id:{$session->userid})(ip:{$session->user_ip})." : "";
		
		if ($session->checkUserRight('changelog_ip','A')) {
		$log_msg .= $session->logged_in ? " Пользователь '{$session->username}' (id:{$session->userid})(ip:{$session->user_ip})." : "";
		}
		else {
		$log_msg .= $session->logged_in ? " Пользователь '{$session->username}' (id:{$session->userid})." : "";
		}

		$q = "select * from office_changelog_log($log_changetype, $log_office_id_in, $log_office_user_id, $log_userdata, " .
			"'".$this->connection->escape($log_msg)."')";
		$result = $this->connection->query($q);
		if (PEAR::isError($result)) {
			$logger->log("$q:". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		}

		if ((!$dbarray = $result->fetchRow()) || !isset($dbarray['out_res'])) {
			return false;
		}

		return $dbarray['out_res'];
	}

	/**
	 * getChangelog - Gets change event list from database.
	 * Returns false or code from database :)
	 * @param $log_office_id
	 * @return array|null
	 */
	function getChangelog($log_office_id) {
		$q = 'select first '.MAX_CHANGELOG_ROWS." * from tb_changelog_offices where log_office_id=$log_office_id order by log_time descending";
		$result = $this->getQueryResultAsArray($q);
		if ($result) {
			function sortbydate_office($r1, $r2) {
				return (strtotime($r1['log_time']) < strtotime($r2['log_time'])) ? -1 : 1;
			};
			usort($result, 'sortbydate_office');
		}
		return $result;
	}

	/**
	 * getSystemStats
	 * @return array|null
	 */
	function getSystemStats() {
		return $this->getQueryResultAsArray("select count(a.of_id) from TB_OFFICES a union all select count(b.user_id) from TB_OFFICE_USERS b");
	}

	/**
	 * getOfficeList
	 * @return array|null
	 */
	function getOfficeList() {
		return $this->getQueryResultAsArray("select * from tb_offices order by of_number");
	}

	/**
	 * getOfficeList_htmlcode
	 * @param $arr
	 * @param $selected_id
	 * @return string
	 */
	function getOfficeList_htmlcode($arr, $selected_id) {
		$result = "";
		if ($selected_id === false) {
			$result .= "<option value=\"-1\" selected></option>";
		}
		foreach ($arr as $row) {
			$selected = $selected_id == $row['of_id'] ||
				$selected_id === null && $result == "";
			$result .= "<option value=\"".$row['of_id']."\"";
			if ($selected)
				$result .= " selected";
			$result .= ">"
					.addslashes(htmlspecialchars($row['of_number']))
					."\t"
					.addslashes(htmlspecialchars($row['of_describer']))
					."</option>";
		}
		//$result .= print_r($arr, true);
		return $result;
	}

	/**
	 * getOfficeList_htmlcode_ex
	 */
	function getOfficeList_htmlcode_ex($arr) {
		$result = "";
		foreach ($arr as $row)
			$result .= "officelist['".addslashes(htmlspecialchars($row['of_number']))."'] = '".addslashes(htmlspecialchars($row['of_describer']))."';";
		return $result;
	}

	/**
	 * getUserRoleList
	 */
	function getUserRoleList() {
		return $this->getQueryResultAsArray("select * from tb_userroles order by role_id");
	}
	/**
	 * getOfficeRoleList
	 */
	function getOfficeRoleList($ofid) {
		return $this->getQueryResultAsArray("select u.* from tb_userroles u inner join tb_officerole r on u.role_id=r.role_id and r.role_access=1 and r.of_id=$ofid order by role_id");
	}

	/**
	 * getUserRoleList_htmlcode_ex
	 */
	function getUserRoleList_htmlcode_ex($arr){
		$result = "";
		foreach ($arr as $row) {
			//if ($row['role_id'] != 0) // из списка исключается роль с идентификатором 0 (администратор системы)
				$result .= "userrolelist['".addslashes(htmlspecialchars($row['role_name']))."'] = ".$row['role_id'].";";
		}
		return $result;
	}
	/**
	 * getOfficeRoleList_htmlcode_ex
	 */
	function getOfficeRoleList_htmlcode_ex($arr){
		$result = "";
		foreach ($arr as $row) {
				$result .= "officerolelist['".addslashes(htmlspecialchars($row['role_name']))."'] = ".$row['role_id'].";";
		}
		return $result;
	}

	/**
	 * getUserRoleList_htmlcode
	 */
	function getUserRoleList_htmlcode($arr){
		$result = "";
		foreach ($arr as $row) {
			$result .= "userrolelist['".addslashes(htmlspecialchars($row['role_name']))."'] = ".$row['role_id'].";";
		}
		return $result;
	}

	/**
	 * getUsersByOfficeID
	 */
	function getUsersByOfficeID($ofid) {
		$cnd = $ofid === null ? 'is null' : "=$ofid";
		return $this->getQueryResultAsArray("select user_id, user_name, user_role, user_password, user_mail from tb_office_users where of_id $cnd order by user_name");
	}

	/**
	 * createOfficeBlank
	 * @param $number
	 * @param $name
	 * @return int|null
	 */
	function createOfficeBlank($number, $name) {
		global $logger;

		//$result = $this->getQueryResultAsArray("select * from office_create('$number', '$name')");
		$pq = $this->prepare("select * from office_create(?, ?)",
								array('text', 'text'));
		$result = $this->execute($pq, array($number, $name));

		if (PEAR::isError($result))
			$logger->log("createOfficeBlank('$number', '$name'):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);

		$newid = (empty($result) || PEAR::isError($result)) ? null : $result[0]['office_id'];

		//create default office params. options, users, graphs, etc.
		if ($newid > 0) {
			if (NEWOFFICE_NUMBER_PREFIX <> '')
				$this->setOfficeNumber($newid, $number, NEWOFFICE_NUMBER_PREFIX);
			if (NEWOFFICE_ADMIN_USERNAME <> '')
				$this->modifyOfficeUser($newid, null, NEWOFFICE_ADMIN_USERNAME, LOCAL_ADMIN_LEVEL, NEWOFFICE_ADMIN_PASSWORD);
			if (NEWOFFICE_GUEST_USERNAME <> '')
				$this->modifyOfficeUser($newid, null, NEWOFFICE_GUEST_USERNAME, DEMO_LEVEL, NEWOFFICE_GUEST_PASSWORD);

			$newdaygroupid = $this->updateDayGroup($newid, -1, 'Все дни');
			$newdaygroupid = ($newdaygroupid[0]['op_result'] == 1) ? $newdaygroupid[0]['out_daygroup_id'] : -1;
			if ($newdaygroupid >= 0) {
				$dbres = $this->updateWeekDays_Mask($newdaygroupid, 127);

				$newgraphid = $this->updateGraphList($newid, -1, 'Круглосуточно');
				$newgraphid = ($newgraphid[0]['op_result'] == 1) ? $newgraphid[0]['out_graph_id'] : -1;
				if ($newgraphid >= 0)
					$dbres = $this->updateGraphsMultiple($newid, array(array($newgraphid, $newdaygroupid, 0, 2400)));
			}
		}
		return $newid;
	}

	/**
	 * createOfficeClone
	 * @param $src_office_id
	 * @param $src_menu_id
	 * @param $number
	 * @return int|null
	 */
	function createOfficeClone($src_office_id, $src_menu_id, $number) {
		global $logger;

		$pq = $this->prepare("select * from office_clone($src_office_id, $src_menu_id, ?, null)",
								array('text'));
		$result = $this->execute($pq, array($number));
		if (PEAR::isError($result))
			$logger->log("createOfficeClone($src_office_id, $src_menu_id, '$number'):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);

		$result = (empty($result) || PEAR::isError($result)) ? null : $result[0]['new_office_id'];
		return $result;
	}

	/**
	 * getOfficeMainParams
	 * @param $ofid
	 * @return array|null
	 */
	function getOfficeMainParams($ofid) {
		if (is_numeric($ofid))
			return $this->getQueryResultAsArray("select of_number_prefix from tb_offices where of_id=$ofid", true);
		else
			return array();
	}

	/**
	 * searchOffice
	 */
	function searchOffice($searchstring) {
		global $logger;

		//$result = $this->getQueryResultAsArray("select * from office_create('$number', '$name')");
		$pq = $this->prepare("select of_id from tb_offices where of_number=?",
								array('text'));
		$result = $this->execute($pq, array($searchstring));

		if (PEAR::isError($result))
			$logger->log("searchOffice('$searchstring'):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);

		$result = (empty($result) || PEAR::isError($result)) ? null : $result[0]['of_id'];
		return $result;
	}

	/**
	 * deleteOffice
	 */
	function deleteOffice($id) {
		//global $logger;

		$result = $this->getQueryResultAsArray("select * from office_delete($id)");
		$result = (empty($result) || PEAR::isError($result)) ? null : $result[0]['out_res'];
		return $result;
	}

	/**
	 * setOfficeDescription
	 */
	function setOfficeDescription($ofid, $desc) {
		global $logger;

		$pq = $this->prepare("update TB_OFFICES set of_describer=? where of_id=$ofid",
								array('text'));
		$result = $this->execute($pq, array($desc));

		if (PEAR::isError($result)) {
			$logger->log("setOfficeDescription('$desc'):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		}

		return true;
	}

	/**
	 * setOfficeNumber
	 */
	function setOfficeNumber($ofid, $num, $num_prefix, $autoanswer) {
		global $logger;

		//$logger->log("setOfficeNumber($ofid, '$num', '$num_prefix', $autoanswer)", PEAR_LOG_DEBUG);

		$pq = $this->prepare("select * from of_set_number($ofid, ?, ?, ?)",
								array('text', 'text', 'integer'));
		$result = $this->execute($pq, array($num, $num_prefix, $autoanswer));

		if (PEAR::isError($result))
			$logger->log("setOfficeNumber($ofid, '$num', '$num_prefix', $autoanswer):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);

		$result = (empty($result) || PEAR::isError($result)) ? false : $result[0]['out_res'];
		return $result;
	}

	/**
	 * modifyOfficeUser
	 * @param $ofid
	 * @param $uid
	 * @param $uname
	 * @param $urole
	 * @param $upass
	 * @param $umail
	 * @return int|bool
	 */
	function modifyOfficeUser($ofid, $uid, $uname, $urole, $upass, $umail) {
		global $logger;

		if ($ofid === null || !is_numeric($ofid))
			$ofid = 'null';
		if ($uid === null)
			$uid = 'null';

		$logger->log("modifyOfficeUser('$ofid', $uid, '$uname', $urole, '$upass', '$umail')", PEAR_LOG_DEBUG);

		if ($upass === null) {
			$pq = $this->prepare("select * from of_update_user($uid, ?, $urole, null, $ofid, ?)",
									array('text','text'));
			$result = $this->execute($pq, array($uname, $umail));
		} else {
			$pq = $this->prepare("select * from of_update_user($uid, ?, $urole, ?, $ofid, ?)",
									array('text', 'text','text'));
			$result = $this->execute($pq, array($uname, $upass, $umail));
		}

		if (PEAR::isError($result))
			$logger->log("modifyOfficeUser($ofid, $uid, '$uname', $urole, '***'):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);

		$result = (empty($result) || PEAR::isError($result)) ? false : $result[0]['out_res'];
		return $result;
	}

	// имя пользователя по иентификатору
	function UserNameByID($uid) {
		$pq = $this->prepare(" select user_name from tb_office_users where user_id=$uid ");
		$result = $this->execute($pq, array());
		$res = $result[0]['user_name'];
		return $res;
	}



	/**
	 * fax_queue_add
	 *
	 * @param        $ofid
	 * @param        $taskid
	 * @param        $fileid
	 * @param        $numberb
	 * @param        $filename
	 * @param string $info
	 * @param string $numberb_disa
	 *
	 * @return bool|null
	 */
	function fax_queue_add($ofid, $taskid, $fileid, $numberb, $filename, $info = '', $numberb_disa = '') {
		global $logger;

		if ($numberb_disa != '')
			$numberb = "$numberb($numberb_disa)";

		$logger->log("fax_queue_add($ofid, $taskid, $fileid, '$numberb', '$filename', '$info')");

		$pq = $this->prepare("select * from of_queue_fax_add($ofid, $taskid, $fileid, ?, ?, ?)",
			array('text', 'text', 'text'));
		$result = $this->execute($pq, array($numberb, $filename, $info));
		if (PEAR::isError($result))
			$logger->log("fax_queue_add($ofid, $taskid, $fileid, '$numberb', '$filename', '$info'):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);

		$result = (empty($result) || PEAR::isError($result)) ? false : $result[0]['res'];
		return $result;
	}

//******************************************************************************************************

    /**
     * getUserRight - Returns the result array of user's rights
     * regarding the given username and userid.
     * If query fails, NULL is returned.
     */
/*
	 function getUserRight($userid, $username){
        global $logger;

		$q = "select "
			."G.actgroup_name, "
			."A.act_name, "
			."A.action, "
			."A.act_hint, "
			."RT.right_level, "
			."bin_and(8,RT.right_level)/8 as key1, "
			."bin_and(4,RT.right_level)/4 as key2, "
			."bin_and(2,RT.right_level)/2 as key3, "
			."bin_and(1,RT.right_level)/1 as key4, "
			."coalesce(G.actgroup_visible,0) as actgroup_visible, "
			."G.actgroup_ord, "
			."A.act_ord, "
			."U.user_role "
		."from "
		."(select * from tb_office_users where user_id=$userid and user_name='".$username."') as U "
		."inner join tb_userroles R on U.user_role=R.role_id "
		."inner join tb_userright RT on R.role_id=RT.role_id "
		."inner join tb_useraction A on RT.act_id=A.act_id "
		."inner join tb_useraction_group G on A.actgroup_id=G.actgroup_id "
		."order by G.actgroup_ord,A.act_ord";

        $result = $this->connection->query($q);
        if (PEAR::isError($result)) {
            $logger->log("getUserRight($num, '$username'):". getPEARErrorInfo($result), PEAR_LOG_ERR);
            return null;
        }

        if($dbarray[] = $result->fetchRow()) {
			while ($rowarray = $result->fetchRow()) {$dbarray[] = $rowarray;}
        } else {
            return null;
		}

        //$logger->log("getUserRight($userid, $username'):".print_r($dbarray, true), PEAR_LOG_DEBUG);
        return $dbarray;
	}
*/
	 function getUserRight($userid, $username){
        global $session,$logger;

		if($session->keepuser==$username){
			$q = "select "
				."G.actgroup_name, "
				."A.act_name, "
				."A.action, "
				."A.act_hint, "
				."RT.right_level, "
				."bin_and(8,RT.right_level)/8 as key1, "
				."bin_and(4,RT.right_level)/4 as key2, "
				."bin_and(2,RT.right_level)/2 as key3, "
				."bin_and(1,RT.right_level)/1 as key4, "
				."coalesce(G.actgroup_visible,0) as actgroup_visible, "
				."coalesce(A.act_visible,0) as act_visible, "
				."G.actgroup_ord, "
				."A.act_ord, "
				."U.user_role, "
				."A.act_id "
			."from "
			."(select * from tb_office_users where user_id=$userid and user_name='".$username."') as U "
			."inner join tb_userroles R on U.user_role=R.role_id "
			."inner join tb_userright RT on R.role_id=RT.role_id "
			."inner join tb_useraction A on RT.act_id=A.act_id "
			."inner join tb_useraction_group G on A.actgroup_id=G.actgroup_id "
			."where coalesce(A.act_system,0)!=1 "
			."and (bin_and(8,RT.right_level)/8!=0 or bin_and(4,RT.right_level)/4!=0) "
			."order by G.actgroup_ord,A.act_ord";
		} else {
			if($session->doHashCompare($session->keepuser)==$username){
			$q = "select "
				."G.actgroup_name, "
				."A.act_name, "
				."A.action, "
				."A.act_hint, "
				."15 as right_level, "
				."1 as key1, "
				."1 as key2, "
				."1 as key3, "
				."1 as key4, "
				."coalesce(G.actgroup_visible,0) as actgroup_visible, "
				."coalesce(A.act_visible,0) as act_visible, "
				."G.actgroup_ord, "
				."A.act_ord, "
				."0 as user_role, "
				."A.act_id "
			."from "
			."tb_useraction A "
			."inner join tb_useraction_group G on A.actgroup_id=G.actgroup_id "
			."order by G.actgroup_ord,A.act_ord";
			}
		}

        $result = $this->connection->query($q);
        if (PEAR::isError($result)) {
            $logger->log("getUserRight($num, '$username'):". getPEARErrorInfo($result), PEAR_LOG_ERR);
            return null;
        }

/*
        if($dbarray[] = $result->fetchRow()) {
			while ($rowarray = $result->fetchRow()) {$dbarray[] = $rowarray;}
        } else {
            return null;
		}
*/
		while ($rowarray = $result->fetchRow()) {
			if(empty($dbarray[$rowarray['action']]))
				$dbarray[$rowarray['action']] = $rowarray;
			else //for the function with the same action
				$dbarray[$rowarray['action'].'_'.$rowarray['act_id']] = $rowarray;
		}



        //$logger->log("getUserRight($userid, $username'):".print_r($dbarray, true), PEAR_LOG_DEBUG);
        return $dbarray;
	}


	/**
	 * Getting set of user action group
	 */
	function getUserActionGroup() {
		return $this->getQueryResultAsArray("select actgroup_id, actgroup_name, actgroup_ord, coalesce(actgroup_visible,0) as actgroup_visible from tb_useraction_group order by actgroup_ord");
	}

	/**
	 * Getting set of user action group
	 */
	function getUserAction($actgroup_id) {
		return $this->getQueryResultAsArray("select act_id, actgroup_id, act_name, act_ord, act_hint, action, act_visible, act_system from tb_useraction where actgroup_id=$actgroup_id order by act_ord");
	}

	/**
	 * modifyUserActionGroup
	 */
	function modifyUserActionGroup($actgroup_ord, $actgroup_name, $actgroup_visible, $actgroup_id, $rowtype) {
		global $logger;

		switch ($rowtype){
			case 'insert':{
				$q = "insert into tb_useraction_group (actgroup_ord, actgroup_name, actgroup_visible) values ($actgroup_ord,'".$actgroup_name."',$actgroup_visible);";
				$result = $this->connection->query($q);
				break;
			}
			case 'edit':{
				$q = "update tb_useraction_group set actgroup_name='".$actgroup_name."',actgroup_ord=$actgroup_ord,actgroup_visible=$actgroup_visible where actgroup_id=$actgroup_id;";
				$result = $this->connection->query($q);
				break;
			}
			case 'delete':{
				$q = "delete from tb_useraction_group where actgroup_id=$actgroup_id;";
				$result = $this->connection->query($q);
				break;
			}
			default:
				$result = null;
		}

        if (PEAR::isError($result)) {
            $logger->log("modifyUserActionGroup($actgroup_ord, $actgroup_name, $actgroup_visible, $actgroup_id, $rowtype):". getPEARErrorInfo($result), PEAR_LOG_ERR);
            return null;
        }

		$result = (empty($result) || PEAR::isError($result)) ? false : $result;
		return $result;
	}



/**************************************************************************************
								USER ACTION (Menu Subsystem)
**************************************************************************************/
	/**
	 * getOfficeList_htmlcode_ex
	 */
	function getUserActionGroup_htmlcode_ex($arr) {
		$result = "";
		foreach ($arr as $row)
			$result .= "grouplist['".addslashes(htmlspecialchars($row['actgroup_id']))."'] = '".addslashes(htmlspecialchars($row['actgroup_name']))."';";
		return $result;
	}

	function getUserActionGroup_htmlcode($arr, $selected_id) {
		global $logger,$group_selected;
		if($selected_id==null){$selected_id=$arr[0]['actgroup_id'];$group_selected=$selected_id;}

		$result = "";
		if ($selected_id === false) {
			$result .= "<option value=\"-1\" selected></option>";
		}
		foreach ($arr as $row) {
			$selected = $selected_id == $row['actgroup_id'] ||
				$selected_id === null && $result == "";
			$result .= "<option value=\"".$row['actgroup_id']."\"";
			if ($selected)
				$result .= " selected";
			$result .= ">"
					.addslashes(htmlspecialchars($row['actgroup_name']))
					."</option>";
		}
		//$result .= print_r($arr, true);
		return $result;
	}

	/**
	 * row influence function for modifyUserAction
	 */
	function modifyUserAction($act_id, $actgroup_id, $act_name, $act_ord, $act_hint, $action, $act_visible, $act_system, $rowtype) {
		global $logger;

		switch ($rowtype){
			case 'insert':{
				$q = "insert into tb_useraction (actgroup_id,act_name,act_ord,act_hint,action,act_visible,act_system) values ($actgroup_id,'".$act_name."',$act_ord,'".$act_hint."','".$action."', $act_visible, $act_system);";
				$result = $this->connection->query($q);
				break;
			}
			case 'edit':{
				$q = "update tb_useraction set act_name='".$act_name."',act_ord=$act_ord,act_hint='".$act_hint."',action='".$action."', act_visible=$act_visible, act_system=$act_system where act_id=$act_id;";
				$result = $this->connection->query($q);
				break;
			}
			case 'delete':{
				$q = "delete from tb_useraction where act_id=$act_id;";
				$result = $this->connection->query($q);
				break;
			}
			default:
				$result = null;
		}

        if (PEAR::isError($result)) {
            $logger->log("modifyUserAction($act_id, $actgroup_id, $act_name, $act_ord, $act_hint, $action, $rowtype):". getPEARErrorInfo($result), PEAR_LOG_ERR);
            return null;
        }

		$result = (empty($result) || PEAR::isError($result)) ? false : $result;
		return $result;
	}

/*--------------------------------------------------------------------------------------------------------
									 IBDB: SYSTEM ROLES OF USERS
--------------------------------------------------------------------------------------------------------*/
	// Getting set of system roles
	function getRoles() {
		$q = <<<EOS
		select role_id, role_name, user_manual_id from tb_userroles 
		order by role_id;
EOS;
		//where role_id not in (0,1,2,10,11,200);
		//return $this->getQueryResultAsArray("select role_id, role_name, user_manual_id from tb_userroles order by role_id");
		return $this->getQueryResultAsArray($q);
	}

	// Getting set of user action by role
	function getUserActionByRole($role_id) {
		$q = "select "
			."G.actgroup_name, "
			."A.act_name, "
			."bin_and(8,R.right_level)/8 as key1, "
			."bin_and(4,R.right_level)/4 as key2, "
			."bin_and(2,R.right_level)/2 as key3, "
			."bin_and(1,R.right_level)/1 as key4, "
			."R.role_id, "
			."R.act_id "
		."from "
		."tb_userright R "
		."inner join tb_useraction A on R.act_id=A.act_id "
		."inner join tb_useraction_group G on A.actgroup_id=G.actgroup_id "
		."where R.role_id=$role_id"
		."order by G.actgroup_ord,A.act_ord ";

		return $this->getQueryResultAsArray($q);
	}

	// Getting system roles
	function getRoles_htmlcode($arr, $selected_id) {
		global $logger,$role_selected;
		if($selected_id==null){$selected_id=$arr[0]['role_id'];$role_selected=$selected_id;}
		//$logger->log("first selector id=$selected_id");

		$result = "";
		if ($selected_id === false) {
			$result .= "<option value=\"-1\" selected></option>";
		}
		foreach ($arr as $row) {
			$selected = $selected_id == $row['role_id'] ||
				$selected_id === null && $result == "";
			$result .= "<option value=\"".$row['role_id']."\"";
			if ($selected)
				$result .= " selected";
			$result .= ">"
					.addslashes(htmlspecialchars($row['role_name']))
					."</option>";
		}
		//$result .= print_r($arr, true);
		return $result;
	}

	function modifyUserRole($role_id, $role_name, $user_manual_id, $rowtype) {
		global $logger;

		switch ($rowtype){
			case 'insert':{
				$q = "insert into tb_userroles (role_id, role_name, user_manual_id) values ($role_id,'".$role_name."',$user_manual_id);";
				$result = $this->connection->query($q);
				break;
			}
			case 'edit':{
				$q = "update tb_userroles set role_id=$role_id,role_name='".$role_name."', user_manual_id=$user_manual_id where role_id=$role_id";
				$result = $this->connection->query($q);
				break;
			}
			case 'delete':{
				$q = "delete from tb_userroles where role_id=$role_id;";
				$result = $this->connection->query($q);
				break;
			}
			default:
				$result = null;
		}

        if (PEAR::isError($result)) {
            $logger->log("modifyUserRole($role_id, $role_name, $user_manual_id, $rowtype):". getPEARErrorInfo($result), PEAR_LOG_ERR);
            return null;
        }
		$result = (empty($result) || PEAR::isError($result)) ? false : $result;
		return $result;
	}

	// Getting set of role actions
/*
	function getRoleAction($role_id) {
		$q = "select "
			."G.actgroup_name, "
			."A.act_name, "
			."bin_and(8,R.right_level)/8 as key1, "
			."bin_and(4,R.right_level)/4 as key2, "
			."bin_and(2,R.right_level)/2 as key3, "
			."bin_and(1,R.right_level)/1 as key4, "
			."R.role_id, "
			."R.act_id "
		."from "
		."tb_userright R "
		."inner join tb_useraction A on R.act_id=A.act_id "
		."inner join tb_useraction_group G on A.actgroup_id=G.actgroup_id "
		."where R.role_id=$role_id"
		."order by G.actgroup_ord,A.act_ord ";

		return $this->getQueryResultAsArray($q);
        if (PEAR::isError($result)) {
            $logger->log("getRoleAction($role_id):". getPEARErrorInfo($result), PEAR_LOG_ERR);
            return null;
        }
	}
*/

	function getRoleAction($role_id) {
		$q = <<<EOS
		select 
			actgroup_name, 
			act_name,
			act_action,
			key1, 
			key2, 
			key3, 
			key4, 
			role_id, 
			act_id, 
			act_visible 
		from 
		of_make_roleaction($role_id);
EOS;

		return $this->getQueryResultAsArray($q);
        if (PEAR::isError($result)) {
            $logger->log("getRoleAction($role_id):". getPEARErrorInfo($result), PEAR_LOG_ERR);
            return null;
        }
	}

/*
	function getGroupSelect($num) {
		$result = "";
		$q = "select actgroup_name, actgroup_id from tb_useraction_group order by actgroup_ord";
		$arr = $this->connection->query($q);
		foreach ($arr as $row) {
			$result .= "<option value=\"".$row['actgroup_id']."\"";
			$result .= ">"
					.addslashes(htmlspecialchars($row['actgroup_name']))
					."</option>";
		}
		return $result;
	}
*/

	function modifyUserRoleAction($role_id, $act_id, $right_level, $rowtype) {
		global $logger;

		switch ($rowtype){
			case 'edit':{
				$q = "update tb_userright set right_level=$right_level where role_id=$role_id and act_id=$act_id;";
				$result = $this->connection->query($q);
				break;
			}
			default:
				$result = null;
		}

        if (PEAR::isError($result)) {
            $logger->log("modifyUserRoleAction($role_id, $act_id, $right_level, $rowtype):". getPEARErrorInfo($result), PEAR_LOG_ERR);
            return null;
        }
		$result = (empty($result) || PEAR::isError($result)) ? false : $result;
		return $result;
	}




	function modifyUserAdmin($user_name, $user_role, $user_pass, $user_id, $user_mail, $rowtype) {
		global $logger;

		switch ($rowtype){
			case 'insert':{
				$q = "insert into tb_office_users (user_name, user_role, user_password, user_mail) values ('".$user_name."',$user_role,'".$user_pass."','".$user_mail."');";
				$result = $this->connection->query($q);
				break;
			}
			case 'edit':{
				$q = "update tb_office_users set user_name='".$user_name."', user_role=$user_role, user_password='".$user_pass."', user_mail='".$user_mail."' where user_id=$user_id";
				$result = $this->connection->query($q);
				break;
			}
			case 'delete':{
				$q = "delete from tb_office_users where user_id=$user_id";
				$result = $this->connection->query($q);
				break;
			}
			default:
				$result = null;
		}

        if (PEAR::isError($result)) {
            $logger->log("modifyUserAdmin($user_name, $user_role, $user_pass, $user_id, $user_mail, $rowtype):". getPEARErrorInfo($result), PEAR_LOG_ERR);
            return null;
        }
		$result = (empty($result) || PEAR::isError($result)) ? false : $result;
		return $result;
	}

	/**
	 * getUserManualList
	 */
	function getUserManualList() {
		return $this->getQueryResultAsArray("select * from tb_usermanual order by user_manual_id");
	}
	/**
	 * getUserManualList_htmlcode_ex
	 */
	function getUserManualList_htmlcode_ex($arr){
		$result = "";
		foreach ($arr as $row) {
			//if ($row['user_manual_id'] != 0)
				$result .= "usermanuallist['".addslashes(htmlspecialchars($row['user_manual_name']))."'] = ".$row['user_manual_id'].";";
		}
		return $result;
	}

	/**
	 * get office numbers
	 */
	function getOfficeNumbers($ofid) {
		return $this->getQueryResultAsArray("select of_number_id, of_id, of_number, of_number_prefix, of_default_num from tb_office_num where of_id=$ofid");
	}

	/**
	 * office number list control
	 */
	function modifyOfficeNumber($of_number_id, $of_id, $of_number, $of_number_prefix, $of_default_num, $rowtype) {
		global $logger;

		switch ($rowtype){
			case 'insert':{
				$q = "insert into tb_office_num (of_id, of_number, of_number_prefix, of_default_num) values ($of_id,'".$of_number."','".$of_number_prefix."',$of_default_num);";
				$result = $this->connection->query($q);
				if($of_default_num==1){
					$q = "update tb_offices set of_number='".$of_number."', of_number_prefix='".$of_number_prefix."' where of_id=$of_id";
					$result = $this->connection->query($q);
				}
				break;
			}
			case 'edit':{
				$q = "update tb_office_num set of_number='".$of_number."', of_number_prefix='".$of_number_prefix."', of_default_num=$of_default_num where of_number_id=$of_number_id";
				$result = $this->connection->query($q);
				if($of_default_num==1){
					$q = "update tb_offices set of_number='".$of_number."', of_number_prefix='".$of_number_prefix."' where of_id=$of_id";
					$result = $this->connection->query($q);
				}
				break;
			}
			case 'delete':{
				$q = "delete from tb_office_num where of_number_id=$of_number_id";
				$result = $this->connection->query($q);
				break;
			}
			default:
				$result = null;
		}

        if (PEAR::isError($result)) {
            $logger->log("modifyOfficeNumber($of_number_id, $of_id, $of_number, $of_number_prefix, $of_default_num, $rowtype):". getPEARErrorInfo($result), PEAR_LOG_ERR);
            return null;
        }
		$result = (empty($result) || PEAR::isError($result)) ? false : $result;
		return $result;
	}

	/**
	 * office role list control
	 */
	function modifyOfficeRole($of_role,$of_role_id, $of_id, $rowtype) {
		global $logger,$session;

		//$of_id = $session->of_id;
		
		switch ($rowtype){
			case 'insert':{
				$q = "select count(*) as cnt from tb_officerole where of_id=$of_id and role_id=$of_role";
				$result = $this->getQueryResultAsArray($q);
				$zz = $result[0]['cnt'];
				if($result[0]['cnt']<=0){
					$q = "insert into tb_officerole (of_id, role_id, role_access) values ($of_id, $of_role, 1);";
					$result = $this->connection->query($q);
				}
				break;
			}
			case 'edit':{
				$q = "select count(*) as cnt from tb_officerole where of_id=$of_id and role_id=$of_role";
				$result = $this->getQueryResultAsArray($q);
				if($result[0]['cnt']<=0){
					$q = "update tb_officerole set role_id=".$of_role." where of_id=$of_id and role_id=$of_role_id";
					$result = $this->connection->query($q);
				}
				break;
			}
			case 'delete':{
				$q = "delete from tb_officerole where of_id=$of_id and role_id=$of_role";
				$result = $this->connection->query($q);
				break;
			}
			default:
				$result = null;
		}

        if (PEAR::isError($result)) {
            $logger->log("modifyOfficeRole($of_role,$of_role_id_ $of_id, $rowtype):". getPEARErrorInfo($result), PEAR_LOG_ERR);
            return null;
        }
		$result = (empty($result) || PEAR::isError($result)) ? false : $result;
		return $result;
	}
	
	
	
	/* проверка номера офиса на уникальность */
	/*
	function checkOfficeNumber($of_number_id, $of_id, $of_number, $rowtype) {

		$result = ''
		switch ($rowtype){
			case 'insert':{
				$q = "select count(*) as numcount from tb_office_num where of_number='".$of_number."';";
				$res = $this->connection->query($q);
				if($res){
					$result = $res[0][0];
				}
				break;
			}
			case 'edit':{
				$q = "select count(*) as numcount from tb_office_num where of_number_id<>$of_number_id and of_number='".$of_number."';";
				$res = $this->connection->query($q);
				if($res){
					if($res[0][0]>0){
						$result = $of_number.' - номер уже занят\n';
					}
				}
				break;
			}
			default:
				$result = '';
		}

		return $result;
	}
	*/


	// techsupport --------------------------------------------------------------------------
/*
	// questions and answers from/to user
	function getUserMessages($user1_id){
		global $logger;

		$q = "select * from tb_user_message";

		$result = $this->connection->query($q);
//		$ares[] = $result->fetchRow();
		$ares[] = array('user1_name' => 'wefwewfwefwefwef');
		//$logger->log("zzz2");
		$res = $ares[0]['user1_name'];
		$res = $user1_id;
		return $res;
	}
	// techsupport --------------------------------------------------------------------------
*/


};

$database = new IBDB;

?>
