<?php
require_once(dirname(__FILE__).'/constants_dynmenu.php');
require_once(dirname(__FILE__).'/constants_changelog.php');
require_once(dirname(__FILE__).'/session.php');
require_once 'MDB2.php';
PEAR::setErrorHandling(PEAR_ERROR_RETURN);

/**
 * getPEARErrorInfo - Returns full info from PEAR::Error
 */
if (!function_exists('getPEARErrorInfo')) {
	function getPEARErrorInfo($dberrobj, $sep = "\n") {
		if (PEAR::isError($dberrobj)) {
			$dbmsg = $dberrobj->getUserInfo();
			$dbmsgd = $dberrobj->getDebugInfo();
			$dbmsgd = (strcmp($dbmsg, $dbmsgd) == 0) ? '' : $sep . "dbg: " . $dbmsgd;
			return "msg: " . $dberrobj->getMessage() . "(" . $dberrobj->getCode() . ")" . $sep
					. "dbgmsg: " . $dbmsg . $dbmsgd;
		} else {
			return "";
		}
	}
}

function db_debug_handler(&$db, $scope, $message, $is_manip = null) {
	global $logger;
	$res = MDB2_defaultDebugOutput($db, $scope, $message, $is_manip);
	$logger->log('dbdebug:'.print_r($res, true));
	return $res;
}

class DYNMENUDB {
	public $connection;
	public $db_ok;

	function DYNMENUDB() {
		global $logger;

		$dsn = DB_ENGINE_DYNMENU.'://'.DB_USER_DYNMENU.':'.DB_PASS_DYNMENU.'@'.DB_HOST_DYNMENU.'/'.DB_FILE_DYNMENU;
		$this->connection = MDB2::factory($dsn, false);
		if (PEAR::isError($this->connection)) {
			$this->db_ok = false;
			$logger->log('DYNMENUDB Fatal Error:'. getPEARErrorInfo($this->connection));
			return;
		}
		$this->db_ok = true;

		//$this->connection->setOption('debug', true);
		//$this->connection->setOption('debug_handler', 'db_debug_handler');
		$this->connection->setOption('field_case', CASE_LOWER);
		$this->connection->setOption('portability', MDB2_PORTABILITY_NUMROWS | MDB2_PORTABILITY_FIX_CASE);
		$this->connection->setFetchMode(MDB2_FETCHMODE_ASSOC);
		//$this->connection->setFetchMode(MDB2_FETCHMODE_OBJECT);
		$this->connection->LoadModule('Extended');

//		$logger->log("ibase.timestampformat BEFORE=".ini_get("ibase.timestampformat"));
//		if (function_exists('ibase_timefmt')) {
//			ibase_timefmt("%d.%m.%Y", IBASE_TIMESTAMP);
//			ibase_timefmt("%d.%m.%Y", IBASE_DATE);
//		} else {
//			ini_set("ibase.timestampformat", "%d.%m.%Y");
//			//ini_set("ibase.timeformat", "%H:%M:%S");
//			ini_set("ibase.dateformat", "%d.%m.%Y");
//		}
//		$logger->log("ibase.timestampformat AFTER=".ini_get("ibase.timestampformat"));
	}

	function transactionStart($savepointname = null) {
		global $logger;

		$result = false;
		if ($this->connection->supports('transactions') && $this->connection->supports('checkpoints'))
			$result = $this->connection->beginTransaction($savepointname);
		$logger->log("transactionStart($savepointname)=".print_r($result, true));
		return $result;
	}

	function transactionCommit($savepointname = null) {
		global $logger;

		$result = false;
		if ($this->connection->inTransaction())
			$result = $this->connection->commit($savepointname);
		$logger->log("transactionCommit($savepointname)=".print_r($result, true));
		return $result;
	}

	function transactionRollback($savepointname = null) {
		global $logger;

		$result = false;
		if ($this->connection->inTransaction())
			$result = $this->connection->rollback($savepointname);
		$logger->log("transactionRollback($savepointname)=".print_r($result, true));
		return $result;
	}

	/**
	 * @param $cond_text
	 * @return array
	 *
	 *  -->SUBSTITUTION
		OUTNUMBERA
		EmmiterKCID
		AccountID
		<--
		-->SQL
		SELECT status as result from int_add_pinphones ('%OUTNUMBERA%',%EmmiterKCID%,%AccountID%,0)
		<--
	 *
	 */
	function menu_condition_decode($cond_text) {
		//global $logger;

		//preg_match('/^-->SUBSTITUTION\s(?P<condition_data_params>)\s<--\s-->\w{3}\s(?P<condition_data>)\s<--$/m', $cond_text, $matches);
		//$logger->log("menu_condition_decode1:" . $cond_text, PEAR_LOG_DEBUG);
		//$logger->log("menu_condition_decode2:" . print_r($matches, true), PEAR_LOG_DEBUG);

		$result = array('condition_data_params' => '', 'condition_data' => '');

		//the dumb way
		$spos = stripos($cond_text, "-->SUBSTITUTION\r\n");
		if ($spos !== false) {
			$result['condition_data_params'] = substr($cond_text, $spos + strlen("-->SUBSTITUTION\r\n"), stripos($cond_text, "<--\r\n") - strlen("-->SUBSTITUTION\r\n"));

			$stmp = trim(substr($cond_text, $spos + strlen("-->SUBSTITUTION\r\n" . $result['condition_data_params'] . "<--\r\n")));
			if (stripos($stmp, "-->") === 0 && stripos($stmp, "<--") > 0) {
				$result['condition_data'] = substr($stmp, strlen("-->XXX\r\n"), strlen($stmp) - strlen("-->XXX\r\n" . "<--"));
				$result['condition_data'] = preg_replace("/\<\?php\s*\r\n|\r\n\s*\?\>/", "", $result['condition_data']);
			}
			$result['condition_data_params'] = trim($result['condition_data_params']);
			$result['condition_data'] = trim($result['condition_data']);
		}

		return $result;
	}

	function menu_condition_encode($cond_processor, $cond_params, $cond_text) {
		$cond_type = substr($cond_processor, 0, 3);
		$text_prefix = strcasecmp($cond_type, 'PHP') == 0 ? "<?php\r\n" : '';
		$text_postfix = strcasecmp($cond_type, 'PHP') == 0 ? "\r\n?>" : '';
		return "-->SUBSTITUTION\r\n".trim($cond_params)."\r\n<--\r\n-->$cond_type\r\n$text_prefix".trim($cond_text)."$text_postfix\r\n<--";
	}

	/**
	 * getQueryResultAsArray
	 *
	 * @param      $query
	 * @param bool $singleton
	 * @param null $types
	 *
	 * @return array|null
	 */
	function getQueryResultAsArray($query, $singleton = false, $types = null) {
		global $logger;

		$result = $this->connection->queryAll($query, $types);
		if (PEAR::isError($result)) {
			$logger->log("getQueryResultAsArray($query) ERR:" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
		}
		//$logger->log("getQueryResultAsArray($query) RES:" . print_r($result, true), PEAR_LOG_DEBUG);

		if ($singleton === false)
			return $result;
		else {
			if (empty($result))
				return null;
			else
				return $result[0];
		}
	}

	/**
	 * getUserEditableSynths - gets user editable synths list.
	 * Returns array or null in case of failure.
	 * @param $menuid
	 * @return array|null
	 */
	function getUserEditableSynths($menuid) {
		global $logger;

		$q = "select * from menu_get_items_usereditable($menuid)";
		$result = $this->getQueryResultAsArray($q);
		if (PEAR::isError($result)) {
			$logger->log("getUserEditableSynths($menuid):ERR=". getPEARErrorInfo($result));
			return null;
		} else
			return $result;
	}

	/**
	 * addSynth - Registers global synth in the database.
	 * Returns new synth info or NULL in case of failure.
	 * @param $synth_id
	 * @param $place
	 * @param $menuid
	 * @param $synth_description
	 * @return array|null
	 */
	function addSynth($synth_id, $place, $menuid, $synth_description) {
		global $logger;

		$q = "select * from menu_add_synthez('$synth_id', $place, $menuid, '".$this->connection->escape($synth_description)."')";
		$result = $this->connection->query($q);
		if (PEAR::isError($result)) {
			$logger->log("$q:". getPEARErrorInfo($result));
			return null;
		}

		if ((!$dbarray = $result->fetchRow()) || !$dbarray['id']) {
			return null;
		}

		return $dbarray;
	}

	/**
	 * deleteSynth - Deletes global synth from the database.
	 * Returns deleted synth info or NULL in case of failure.
	 *
	 * @param $synth_id
	 * @param $place
	 * @param $menuid
	 *
	 * @return int|null
	 */
	function deleteSynth($synth_id, $place, $menuid) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		$q = "select * from menu_del_synthez('$synth_id', $place, $menuid)";
		$result = $this->connection->query($q);
		if (PEAR::isError($result)) {
			$logger->log("$q:". getPEARErrorInfo($result));
			return null;
		}

		if ((!$dbarray = $result->fetchRow()) || !$dbarray['id']) {
			return -1;
		}

		return $dbarray['res'];
	}

	/**
	 * getMenuInfo - Gets menu description by menuid.
	 * Returns '' in case of failure.
	 * @param $menuid
	 * @return null|string
	 */
	function getMenuInfo($menuid) {
		global $logger;

		$q = "select * from menu_get_name($menuid)";
		$result = $this->connection->query($q);
		if (PEAR::isError($result)) {
			$logger->log("$q:". getPEARErrorInfo($result));
			return null;
		}

		if((!$dbarray = $result->fetchRow()) || !$dbarray['menu_name']) {
			return '';
		}

		return $dbarray['menu_name'];
	}

	/**
	 * getMenusList - Gets menu list in 'array' format.
	 * Returns null in case of failure.
	 * @return array|null
	 */
	function getMenuList() {
		global $logger;

		$q = 'select * from menu_get_menus_list';
		$result = $this->getQueryResultAsArray($q);
		if (PEAR::isError($result)) {
			$logger->log("getMenusList:ERR=". getPEARErrorInfo($result));
			return null;
		} else
			return $result;
	}

	/**
	 * getMenusListAsOfficeOption - Gets menu list in 'office option' format.
	 * Returns "" in case of failure.
	 * @param $arr_menus
	 * @return string
	 */
	function getMenuListAsOfficeOption($arr_menus) {
		$str = '-1|Сценарий не выбран';
		if (empty($arr_menus))
			return $str;
		foreach ($arr_menus as $menu) {
			if ($str != '')
				$str .= ',';
			$str .= $menu['menu_id'].'|'.$menu['menu_name'];
		}
		return $str;
	}

	/**
	 * cloneMenu - clones specified menu.
	 * Returns new menu id or null
	 * @param $in_menuid
	 * @param $in_menuname
	 * @return null
	 */
	function cloneMenu($in_menuid, $in_menuname) {
		global $database_dynmenu, $logger;

		if ($in_menuid == null || $in_menuname == null)
			return null;

		$pq = $this->prepare("select * from global_menu_duplicate($in_menuid, ?)",
								array('text'));
		$result = $this->execute($pq, array($in_menuname));
		if (PEAR::isError($result))
			$logger->log("cloneMenu($in_menuid, '$in_menuname'):ERR=". getPEARErrorInfo($result));

		$result = (empty($result) || PEAR::isError($result)) ? null : $result[0]['out_menu_id'];
		return $result;
	}

	/**
	 * logMenuChange - Logs change event to database.
	 * Returns false or code from database :)
	 * @param $log_changetype
	 * @param $log_menu_id
	 * @param $log_userdata
	 * @param $log_msg
	 * @return bool
	 */
	function logMenuChange($log_changetype, $log_menu_id, $log_userdata, $log_msg) {
		global $session, $logger;

		//changelog
		$log_office_user_id = $session->logged_in ? $session->userid : -1;
		$log_msg .= $session->logged_in ? " Пользователь '{$session->username}' (id:{$session->userid})." : "";

		$q = "select * from menu_changelog_log($log_changetype, $log_menu_id, $log_office_user_id, $log_userdata, " .
			"'".$this->connection->escape($log_msg)."')";
		$result = $this->connection->query($q);
		if (PEAR::isError($result)) {
			$logger->log("$q:". getPEARErrorInfo($result));
			return false;
		}

		if((!$dbarray = $result->fetchRow()) || !$dbarray['out_res']) {
			return false;
		}

		return $dbarray['out_res'];
	}

	/**
	 * getChangelog - Gets change event list from database.
	 * Returns false or code from database :)
	 * @param $log_menu_id
	 * @return array|null
	 */
	function getChangelog($log_menu_id) {
		$q = 'select first '.MAX_CHANGELOG_ROWS." * from tb_changelog_menus where log_menu_id=$log_menu_id order by log_time descending";
		$result = $this->getQueryResultAsArray($q);
		if ($result) {
			function sortbydate_menus($r1, $r2) {
				return (strtotime($r1['log_time']) < strtotime($r2['log_time'])) ? -1 : 1;
			};
			usort($result, 'sortbydate_menus');
		}
		return $result;
	}

	/**
	 * query - Performs the given query on the database and
	 * returns the result, which may be false, true or a
	 * resource identifier.
	 *
	 * @param $query
	 * @param null $types
	 *
	 * @return mixed
	 */
	function query($query, $types = null) {
		return $this->connection->query($query, $types);
	}

	function destroyLOB($blob) {
		$this->connection->datatype->destroyLOB($blob);
	}

	/**
	 * prepare - wrapper
	 *
	 * @param      $query
	 * @param      $types
	 *
	 * @param null $result_types
	 *
	 * @return object
	 */
	function prepare($query, $types = null, $result_types = null) {
		global $logger;

		$result = $this->connection->prepare($query, $types, $result_types);
		if (PEAR::isError($result)) {
			$logger->log("prepare($query, $types) ERR:\n" . getPEARErrorInfo($result));
			return null;
		}
		return $result;
	}

	/**
	 * execute - wrapper

	 * @param $preparedquery
	 * @param $data

	 * @return object
	 */
	function execute($preparedquery, $data) {
		global $logger;

		if (!$preparedquery) {
			$logger->log("execute ERR0 (called for wrong obj):\n" . print_r($preparedquery, true), PEAR_LOG_ERR);
			return null;
		}

		//$logger->log("execute Q:\n" . print_r($preparedquery, true));
		$result = $preparedquery->execute($data);

		if (PEAR::isError($result)) {
			$logger->log("execute ERR:\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			return null;
		}
		//$logger->log("execute OK:\n" . print_r($result, true));
		if (!isset($preparedquery->is_manip) || !$preparedquery->is_manip) {
			$res = $result->fetchAll();
			$result->free();
		} else
			$res = $result;
		return $res;
	}

};

$database_dynmenu = new DYNMENUDB;

?>
