<?php
require_once(dirname(__FILE__).'/constants.php');
require_once(dirname(__FILE__).'/database.php');
require_once(dirname(__FILE__).'/session.php');
require_once 'MDB2.php';
PEAR::setErrorHandling(PEAR_ERROR_RETURN);

/**
 * getPEARErrorInfo - Returns full info from PEAR::Error
 */
if (!function_exists('getPEARErrorInfo')) {
	function getPEARErrorInfo($dberrobj, $sep = "\n") {
		if (PEAR::isError($dberrobj)) {
			$dbmsg = $dberrobj->getUserInfo();
			$dbmsgd = $dberrobj->getDebugInfo();
			$dbmsgd = (strcmp($dbmsg, $dbmsgd) == 0) ? '' : $sep . "dbg: " . $dbmsgd;
			return "msg: " . $dberrobj->getMessage() . "(" . $dberrobj->getCode() . ")" . $sep
					. "dbgmsg: " . $dbmsg . $dbmsgd;
		} else {
			return "";
		}
	}
}

class TARDB {
	public $connection;
	public $db_ok;

	function TARDB() {
		global $logger;

		$dsn = DB_ENGINE_TAR.'://'.DB_USER_TAR.':'.DB_PASS_TAR.'@'.DB_HOST_TAR.'/'.DB_FILE_TAR;
		$options = array ('debug' => 0);
		$this->connection = MDB2::factory($dsn, $options);
		if (PEAR::isError($this->connection)) {
			$this->db_ok = false;
			$logger->log("TARDB Fatal Error:". getPEARErrorInfo($this->connection));
			return;
		}
		$this->db_ok = true;

		
		$this->connection->setOption('field_case', CASE_LOWER);
		$this->connection->setOption('portability', MDB2_PORTABILITY_NUMROWS | MDB2_PORTABILITY_FIX_CASE);
		$this->connection->setFetchMode(MDB2_FETCHMODE_ASSOC);
		//$this->connection->setFetchMode(MDB2_FETCHMODE_OBJECT);
		$this->connection->LoadModule('Extended');
	}

	/**
	 * getQueryResultAsArray
	 *
	 * @param      $query
	 * @param bool $singleton
	 *
	 * @return array|null
	 */
	function getQueryResultAsArray($query, $singleton = false) {
		global $logger;

		//$logger->log("getQueryResultAsArray($query)" , PEAR_LOG_DEBUG);

		$result = $this->connection->queryAll($query);
		if (PEAR::isError($result)) {
			$logger->log("getQueryResultAsArray($query) ERR:". getPEARErrorInfo($result));
			$result = null;
		}
		//$logger->log("getQueryResultAsArray($query) RES:" . print_r($result, true), PEAR_LOG_DEBUG);

		if ($singleton === false)
			return $result;
		else {
			if (empty($result))
				return null;
			else
				return $result[0];
		}
	}

	/**
	 * query
	 *
	 * @param $query
	 *
	 * @return object
	 */
	function query($query) {
		return $this->connection->query($query);
	}

	/**
	 * prepare - wrapper
	 *
	 * @param      $query
	 * @param      $types
	 *
	 * @param null $result_types
	 *
	 * @return object
	 */
	function prepare($query, $types = null, $result_types = null) {
		global $logger;

		$result = $this->connection->prepare($query, $types, $result_types);
		if (PEAR::isError($result)) {
			$logger->log("prepare($query, $types) ERR:\n" . getPEARErrorInfo($result));
			return null;
		}
		return $result;
	}

	/**
	 * execute - wrapper
	 *
	 * @param $preparedquery
	 * @param $data
	 *
	 * @return array
	 */
	function execute($preparedquery, $data) {
		global $logger;

		if (!$preparedquery) {
			$logger->log("execute ERR0 (called for wrong obj):\n" . print_r($preparedquery, true), PEAR_LOG_ERR);
			return null;
		}

		//$logger->log("execute Q:\n" . print_r($preparedquery, true));
		$result = $preparedquery->execute($data);

		if (PEAR::isError($result)) {
			$logger->log("execute ERR:\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			return null;
		}
		//$logger->log("execute OK:\n" . print_r($result, true));
		if (!isset($preparedquery->is_manip) || !$preparedquery->is_manip) {
			$res = $result->fetchAll();
			$result->free();
		} else
			$res = $result;
		return $res;
	}

	/**
	 * get_stats_geographys_htmlcode
	 *
	 * @param $arr
	 * @param $selected_id
	 *
	 * @return string
	 */
	function get_stats_geographys_htmlcode($arr, $selected_id) {
		$result = '';
		foreach ($arr as $row) {
			$result .= '<option ';
			if ($selected_id == $row['geographyname'])
				$result .= ' selected';
			$result .= '>' . addslashes(htmlspecialchars($row['geographyname'])) . '</option>';
		}
		return $result;
	}

	/**
	 * get_stats_geographys
	 * @return array|null
	 */
	function get_stats_geographys() {
		global $session;

		$number = DB_TAR_DEBUG ? DB_TAR_DEBUG_NUMBER : DB_TAR_FEDERAL_PREFIX . $session->ofnumber;
		//$starttm = date("d.m.Y", mktime(0, 0, 0, date("m"), date("d"), date("Y") - 2));
		$starttm = date("d.m.Y", mktime(0, 0, 0, date("m") , date("d") - 1, date("Y") ));
		$endtm = date("d.m.Y H:i");

		$q = "select distinct geographyname from int_reportaccountmtr('$number', null, '$starttm', '$endtm', 0, -1, -1) order by geographyname";
		$result = $this->getQueryResultAsArray($q);
		//$result = array('geographyname' => 'erwgrrbhythnt');
		return $result;
	}

	/**
	 * get_stats
	 *
	 * @param int  $flagsin
	 * @param int  $page
	 * @param int  $resultsperpage
	 * @param null $sortname
	 * @param null $sortorder
	 * @param null $searchname
	 * @param null $searchquery
	 * @param      $totalcnt
	 * @param      $totalvol
	 * @param      $totalpay
	 *
	 * @return array|null
	 */
	function get_stats($flagsin = DB_TAR_SERVICETYPE_ALLTYPES,
					   $page = 1, $resultsperpage = 50,
					   $sortname = null, $sortorder = null,
					   $searchname = null, $searchquery = null,
					   &$totalcnt, &$totalvol, &$totalpay) {
		global $session;

		$filter = $session->filters[$flagsin];

		$number = DB_TAR_DEBUG ? DB_TAR_DEBUG_NUMBER : DB_TAR_FEDERAL_PREFIX . $session->ofnumber;
		$startnum = 0;
		$endnum = -1;
		$starttm = $filter['sf_startdate'] . ' 00:00:00';
		$endtm = $filter['sf_enddate'] . ' 23:59:59';

		if (!$sortname)
			$sortname = 'creattime, sessionid';
		if (!$sortorder)
			$sortorder = 'desc';
		$sort = "order by $sortname $sortorder";

		if (!$page)
			$page = 1;
		if (!$resultsperpage)
			$resultsperpage = 50;
		$start = ($page - 1) * $resultsperpage;
		$limit = "first $resultsperpage skip $start";

		$addwhere = "";
		if ($searchquery) {
			$addwhere = "and $searchname like ";
			$addwhere .= (substr($searchquery, 0, 1) == '^') ?
					"'" . substr($searchquery, 1) . "%'" :
					"'%$searchquery%'";
		}

		$where = sprintf(" where (realvolume is null or realvolume >= %d and realvolume <= %d) ", 60 * $filter['sf_startvol'], 60 * $filter['sf_endvol']);
		if (in_array($flagsin, array(DB_TAR_SERVICETYPE_ALLTYPES, DB_TAR_SERVICETYPE_MTR, DB_TAR_SERVICETYPE_VOICE, DB_TAR_SERVICETYPE_FAX)) &&
			!empty($filter['sf_numbera']))
			$where .= sprintf(" and numbera like '%%'||'%s'||'%%' ", $filter['sf_numbera']);
		if (in_array($flagsin, array(DB_TAR_SERVICETYPE_ALLTYPES, DB_TAR_SERVICETYPE_MTR, DB_TAR_SERVICETYPE_VOICE_SENT, DB_TAR_SERVICETYPE_FAX_SENT)) &&
			!empty($filter['sf_numberb']))
			$where .= sprintf(" and numberb like '%%'||'%s'||'%%' ", $filter['sf_numberb']);
		if (in_array($flagsin, array(DB_TAR_SERVICETYPE_ALLTYPES, DB_TAR_SERVICETYPE_MTR))) {
			if (!empty($filter['sf_geo']))
				$where .= sprintf(" and geographyname like '%%'||'%s'||'%%' ", strtoupper($filter['sf_geo']));
			/*if ($filter['sf_calltype100'] != '1')
				$where .= " and calltype <> 100 ";
			if ($filter['sf_calltype101'] != '1')
				$where .= " and calltype <> 101 ";
			if ($filter['sf_calltype301'] != '1')
				$where .= " and calltype <> 301 ";*/
				// без входящих
				if ($filter['sf_calltype301'] <> '1')
					$where .= " and calltype <> 301 ";
				// без исходящиx
				if ($filter['sf_calltype201'] <> '1')
					$where .= " and calltype <> 201 ";
				// без пропущенных
				if ($filter['sf_calltype100'] <> '1')
					$where .= " and calltype <> 100 ";

		}

		$qcnt = "select count(creattime) as totalcnt, cast(sum(payablevolume)/60 as integer) as totalvol, sum(pay) as totalpay from INT_REPORTACCOUNTMTR('$number', null, '$starttm', '$endtm', $startnum, $endnum, $flagsin) $where $addwhere";
		$cntres = $this->getQueryResultAsArray($qcnt, true);
		$totalcnt = empty($cntres) ? 0 : $cntres['totalcnt'];
		if ($totalcnt > 0) {
			$totalvol = $cntres['totalvol'];
			$totalpay = $cntres['totalpay'];
			$q = "select $limit * from INT_REPORTACCOUNTMTR('$number', null, '$starttm', '$endtm', $startnum, $endnum, $flagsin) $where $addwhere $sort";
			return $this->getQueryResultAsArray($q);
		} else
			return array();
	}



	function get_name_owner($filter) {
	global $session, $logger;

		$filter['f_fio'] = trim($filter['f_fio']);
		$filter['f_address'] = trim($filter['f_address']);
		$filter['f_docum'] = trim($filter['f_docum']);
		$conditions = '';
		$conditions .= empty($filter['f_fio']) ? '' : ' lower(g.fio) like lower(\'%'.$filter['f_fio'].'%\')';
		$conditions .= empty($filter['f_address']) ? '' : (empty($conditions) ? '' : ' and ').'lower(a.address) like lower(\'%'.$filter['f_address'].'%\')';
		$conditions .= empty($filter['f_docum']) ? '' : (empty($conditions) ? '' : ' and ').'lower(g.document) like lower(\'%'.$filter['f_docum'].'%\')';
		$conditions = 'where '.(empty($conditions) ? ' 1<>1' : $conditions);
		
		$q = "select trim(g.accountgts) as accountgts,trim(g.fio) as fio, a.notify_address as address, document, emitterkcid, accountid from tbgts g inner join tbaccounts a on a.accountid=g.accountid $conditions ";
		return $this->getQueryResultAsArray($q);
	}

	function get_account_owner($filter) {
	global $session, $logger;

		$filter['f_account'] = trim($filter['f_account']);
		$filter['f_number'] = trim($filter['f_number']);
		$conditions = '';
		$conditions .= empty($filter['f_account']) ? '' : ' lower(g.accountgts) like lower(\'%'.$filter['f_account'].'%\')';
		$conditions .= empty($filter['f_number']) ? '' : (empty($conditions) ? '' : ' and ').'lower(p.numbera) like lower(\'%'.$filter['f_number'].'%\')';
		$conditions = 'where '.(empty($conditions) ? ' 1<>1' : $conditions);
		
		$q = "select distinct g.accountgts, g.fio, a.notify_address as address, a.emitterkcid, a.accountid from tbgts g inner join tbaccounts a on a.accountid=g.accountid left join tbphones p on a.accountid=p.accountid $conditions ";
		//$logger->log("  $q", PEAR_DEBUG);
		return $this->getQueryResultAsArray($q);
	}


	/**
	 * get_mblist
	 *
	 * @param int  $page
	 * @param int  $resultsperpage
	 * @param null $sortname
	 * @param null $sortorder
	 * @param null $searchname
	 * @param null $searchquery
	 * @param      $totalcnt
	 * @param      $totalvol
	 *
	 * @return array|null
	 */
	function get_mblist($page = 1, $resultsperpage = 50,
					   $sortname = null, $sortorder = null,
					   $searchname = null, $searchquery = null,
					   &$totalcnt, &$totalvol) {
		global $session;

		$filter = $session->filters[DB_TAR_SERVICETYPE_MB];

		$number = $session->isAdmin() ? DB_TAR_MB_SYSTEMBOXCOMMONID : (DB_TAR_DEBUG ? DB_TAR_DEBUG_NUMBER : $session->ofnumber);
		$starttm = $filter['sf_startdate'] . ' 00:00:00';
		$endtm = $filter['sf_enddate'] . ' 23:59:59';

		if (!$sortname)
			$sortname = 'createdate';
		if (!$sortorder)
			$sortorder = 'desc';
		$sort = "order by $sortname $sortorder";

		if (!$page)
			$page = 1;
		if (!$resultsperpage)
			$resultsperpage = 50;
		$start = ($page - 1) * $resultsperpage;
		$limit = "first $resultsperpage skip $start";

		$addwhere = "";
		if ($searchquery) {
			$addwhere = "and $searchname like ";
			$addwhere .= (substr($searchquery, 0, 1) == '^') ?
					"'" . substr($searchquery, 1) . "%'" :
					"'%$searchquery%'";
		}

		$where = '';
		//$where = sprintf(" where (messagesize >= %d and messagesize <= %d) ", $filter['sf_startvol'], $filter['sf_endvol']);
		if (!empty($filter['sf_numbera']))
			$where .= sprintf(" where numbera like '%%'||'%s'||'%%' ", $filter['sf_numbera']);
		/*
			if ($filter['sf_calltype100'] != '1')
				$where .= " and calltype <> 100 ";
			if ($filter['sf_calltype101'] != '1')
				$where .= " and calltype <> 101 ";
			if ($filter['sf_calltype301'] != '1')
				$where .= " and calltype <> 301 ";
		*/

		$qcnt = "select count(createdate) as totalcnt, cast(sum(messagesize) as integer) as totalvol from fgs_getmessagelist_bytime('$number', '$starttm', '$endtm', null) $where $addwhere";
		$cntres = $this->getQueryResultAsArray($qcnt, true);
		$totalcnt = empty($cntres) ? 0 : $cntres['totalcnt'];
		if ($totalcnt > 0) {
			$totalvol = $cntres['totalvol'];
			$q = "select $limit * from fgs_getmessagelist_bytime('$number', '$starttm', '$endtm', null) $where $addwhere $sort";
			return $this->getQueryResultAsArray($q);
		} else
			return array();
	}

	/**
	 * get_mblist_del
	 *
	 * @return array|null
	 */
	function get_mblist_del() {
		global $session;

		$filter = $session->filters[DB_TAR_SERVICETYPE_MB];

		$number = $session->isAdmin() ? DB_TAR_MB_SYSTEMBOXCOMMONID : (DB_TAR_DEBUG ? DB_TAR_DEBUG_NUMBER : $session->ofnumber);
		$starttm = $filter['sf_startdate'] . ' 00:00:00';
		$endtm = $filter['sf_enddate'] . ' 23:59:59';

		$where = '';
		//$where = sprintf(" where (messagesize >= %d and messagesize <= %d) ", $filter['sf_startvol'], $filter['sf_endvol']);
		if (!empty($filter['sf_numbera']))
			$where .= sprintf(" where numbera like '%%'||'%s'||'%%' ", $filter['sf_numbera']);

		$q = "select createdate, recordid, messageid, messagetypeid, flagsout from fgs_getmessagelist_bytime('$number', '$starttm', '$endtm', null) $where";
		return $this->getQueryResultAsArray($q);
	}

	/**
	 * get_mblist_save
	 */
	function get_mblist_save() {
		global $session, $logger;

		$filter = $session->filters[DB_TAR_SERVICETYPE_MB];

		$number = $session->isAdmin() ? DB_TAR_MB_SYSTEMBOXCOMMONID : (DB_TAR_DEBUG ? DB_TAR_DEBUG_NUMBER : $session->ofnumber);
		$starttm = $filter['sf_startdate'] . ' 00:00:00';
		$endtm = $filter['sf_enddate'] . ' 23:59:59';

		$where = '';
		//$where = sprintf(" where (messagesize >= %d and messagesize <= %d) ", $filter['sf_startvol'], $filter['sf_endvol']);
		if (!empty($filter['sf_numbera']))
			$where .= sprintf(" where numbera like '%%'||'%s'||'%%' ", $filter['sf_numbera']);

		$q = "select createdate, recordid, messageid, messagetypeid, flagsout, replace(recordpath,'\','') as recordpath from fgs_getmessagelist_bytime('$number', '$starttm', '$endtm', null) $where";
		//$logger->log("$q", PEAR_LOG_DEBUG);
		return $this->getQueryResultAsArray($q);
	}
	
	
	/**
	 * getActiveCalls
	 *
	 * @param $number
	 * @param null $starttm
	 * @param null $endtm
	 *
	 * @return array|null
	 */
	function getActiveCalls($number, $starttm = null, $endtm = null) {
		//global $logger;

		$result = $this->getAccountInfo($number);
		if ($result) {
			$q = "select first ".MAX_STATISTICS_ROWS." * from get_account_active_session({$result['emitterkcid']}, {$result['accountid']}, '" . DB_TAR_ACTIVESESSIONS_PREFIX . "')";
			$result = $this->getQueryResultAsArray($q);
		}
		//$logger->log(print_r($result, true));

		return $result;
	}

	/**
	 * getAccountInfo
	 *
	 * @param $number
	 *
	 * @return array|null
	 */
	function getAccountInfo($number) {
		//global $logger;

		/*$q = "select accountid, emitterkcid from tbphones where numbera='$number' and authorizationtypeid=0";
		$res = $this->getQueryResultAsArray($q);
		if ($res) {
			$accountinfo['accountid'] = $res['accountid'];
			$accountinfo['emitterkcid'] = $res['emitterkcid'];
		}*/
		$q = 'select ' .
				'tbphones.emitterkcid, ' .
				'tbphones.accountid, ' .
				'tbaccounts.createdate, ' .
				'tbaccounts.fio, ' .
				'tbaccounts.currencyid, ' .
				'tbaccounts.maxconnections, ' .
				'tbcurrency.currencyname, ' .
				'tbaccountsupdate.summ ' .
		'from tbaccountsupdate ' .
			'inner join tbaccounts on (tbaccountsupdate.emitterkcid = tbaccounts.emitterkcid) and (tbaccountsupdate.accountid = tbaccounts.accountid) ' .
			'inner join tbphones on (tbaccounts.emitterkcid = tbphones.emitterkcid) and (tbaccounts.accountid = tbphones.accountid) ' .
			'inner join tbcurrency on (tbaccounts.currencyid = tbcurrency.currencyid) ' .
		"where tbphones.numbera='$number' and tbphones.authorizationtypeid = 0";
		return $this->getQueryResultAsArray($q, true);
	}

	/**
	 * isAccountExist
	 *
	 * @param $number
	 *
	 * @return bool
	 */
	function isAccountExist($number) {
		$q = "select 1 as res from tbphones where numbera='$number' and authorizationtypeid=0";
		$res = $this->getQueryResultAsArray($q, true);
		return ($res['res'] == 1);
	}

	/**
	 * getMB
	 *
	 * @param $number
	 *
	 * @return array
	 */
	function getMB($number) {
		global $logger;

		//$q = "select * from fgs_getmailbox(null, '$number', null)";
		//$res = $this->getQueryResultAsArray($q, true);

		$pq = $this->prepare('select * from fgs_getmailbox(null, ?, null)',
			array('text'));
		$result = $this->execute($pq, array($number));

		if (PEAR::isError($result)) {
			$logger->log("getMB($number):ERR=". getPEARErrorInfo($result));
			return null;
		} else {
			//$logger->log("getMB($number):".print_r($result, true));
			return empty($result) ? null : $result[0];
		}
	}

	/**
	 * getMBFreeSize
	 *
	 * @param $number
	 *
	 * @return int
	 */
	function getMBFreeSize($number) {
		//$q = "select * from fgs_getfreesize((select boxprivateid from tbboxes where boxcommonid = '$number'))";
		//$res = $this->getQueryResultAsArray($q, true);
		$res = $this->getMB($number);
		return empty($res) ? 0 : $res['freesize'];
	}
	
	function getMBWriteType($number){
		$res = $this->getMB($number);
		return empty($res) ? 0 : $res['boxloop'];
	}
	

	/**
	 * get_message_types
	 * @return array|null
	 */
	function getMBMessageTypes() {
		return $this->getQueryResultAsArray('select * from tbmessagetypes');
	}

	/**
	 * addMBMessage
	 *
	 * @param $number
	 * @param $message_typeid
	 * @param $message_size
	 * @param $message_fileid
	 *
	 * @param string $message_source
	 * @internal param $fileid
	 * @internal param $authnumber
	 * @internal param $dialnumber
	 * @internal param $aon_prefix
	 *
	 * @return int|null
	 */
	function addMBMessage($number, $message_typeid, $message_size, $message_fileid, $message_source = DB_TAR_MESSAGESOURCE_NAME) {
		global $logger;

		$q = "insert into tbmessages(messageid, boxprivateid, messagetypeid, createdate, messagestatus, messagesize, numbera, recordid) "
			. "select null, boxprivateid, $message_typeid, null, 1, $message_size, '$message_source', $message_fileid from tbboxes where boxcommonid = '$number'";
		$pq = $this->prepare("insert into tbmessages(messageid, boxprivateid, messagetypeid, createdate, messagestatus, messagesize, numbera, recordid) "
							. "select null, boxprivateid, $message_typeid, null, 1, $message_size, ?, $message_fileid from tbboxes where boxcommonid = ?",
							array('text', 'text'), MDB2_PREPARE_MANIP);
		$result = $this->execute($pq, array($message_source, $number));

		if (PEAR::isError($result) || !is_numeric($result)) {
			$logger->log("addMBMessage($q):ERR=". getPEARErrorInfo($result));
			return null;
		} else {
			$logger->log("addMBMessage($q):".print_r($result, true));
			return intval($result);
		}
	}

	/**
	 * @param $message_fileid
	 * @param $message_messageid
	 * @return int
	 */
	function deleteMBMessage($message_fileid, $message_messageid = -1) {
		$q = "select * from fgs_delete_message($message_messageid, $message_fileid)";
		$res = $this->getQueryResultAsArray($q, true);
		return empty($res) ? -1 : $res['flagsout'];
	}

	/**
	 * addFaxSendTask
	 *
	 * @param $fileid
	 * @param $authnumber
	 * @param $dialnumber
	 * @param $aon_prefix
	 * @param string $dialnumber_disa
	 *
	 * @return int|null
	 */
	function addFaxSendTask($fileid, $authnumber, $dialnumber, $aon_prefix, $dialnumber_disa = '') {
		global $logger;

		if ($dialnumber_disa != '')
			$dialnumber = "$dialnumber($dialnumber_disa)";
		$faxparams = "Def.FileID=$fileid\r\nInCall.NumberIn=$authnumber\r\nDef.FaxDialNumber=$dialnumber\r\nInCall.AON={$aon_prefix}{$authnumber}\r\n".DB_TAR_TM_ADDPARAMS_FAXSEND;
		$q = "select * from tm_create_project_task(".DB_TAR_TM_PROJECT_ID_FAXSEND.", 0, null, '$dialnumber', ".DB_TAR_TM_TASK_TRY_COUNT_FAXSEND.", '$authnumber', '$faxparams')";
		$pq = $this->prepare('select * from tm_create_project_task('.DB_TAR_TM_PROJECT_ID_FAXSEND.", 0, null, ?, ".DB_TAR_TM_TASK_TRY_COUNT_FAXSEND.", ?, ?)",
								array('text', 'text', 'text'));  //array('text', 'text', 'blob'));
		$result = $this->execute($pq, array($dialnumber, $authnumber, $faxparams));

		if (PEAR::isError($result)) {
			$logger->log("addFaxSendTask($q):ERR=". getPEARErrorInfo($result));
			return null;
		} else {
			$logger->log("addFaxSendTask($q):".print_r($result, true));
			return intval($result[0]['tm_task_id']);
		}
	}

	/**
	 * updateLowBalanceNotifyTask
	 * @param $authnumber
	 * @param $summ
	 * @param $mail
	 * @param $tel
	 * @param $text
	 * @param bool $removetask
	 * @return int|bool|null
	 */
	function updateLowBalanceNotifyTask($authnumber, $summ, $mail, $tel, $text, $removetask = false) {
		global $logger;

		if ($removetask) {
			$q = "delete from tm_task_list where TM_TASK_AUTH_ID='$authnumber' and TM_TASK_PROJECT_ID=".DB_TAR_TM_PROJECT_ID_LOWBALANCENOTIFY;
			$pq = $this->prepare("delete from tm_task_list where TM_TASK_AUTH_ID=? and TM_TASK_PROJECT_ID=".DB_TAR_TM_PROJECT_ID_LOWBALANCENOTIFY,
									array('text'));
			$result = $this->execute($pq, array($authnumber));
		} else {
			$ai = $this->getAccountInfo($authnumber);
			$emitterkcid = $ai['emitterkcid'];
			$accountid = $ai['accountid'];
			$extparams = "User.emitterkcid=$emitterkcid\r\nUser.accountid=$accountid\r\nUser.Summ=$summ\r\nUser.Mail=$mail\r\nUser.Tel=$tel\r\n".DB_TAR_TM_ADDPARAMS_LOWBALANCENOTIFY;//User.Text=$text\r\n

			$task = $this->get_tm_task_props_by_authnumber($authnumber, DB_TAR_TM_PROJECT_ID_LOWBALANCENOTIFY);
			if ($task == null) {
				//create new task
				$q = "select * from tm_create_project_task(".DB_TAR_TM_PROJECT_ID_LOWBALANCENOTIFY.", 0, cast('now' as date)+1/8640, null, ".DB_TAR_TM_TASK_TRY_COUNT_LOWBALANCENOTIFY.", '$authnumber', '$extparams')";
				$pq = $this->prepare('select * from tm_create_project_task('.DB_TAR_TM_PROJECT_ID_LOWBALANCENOTIFY.", 0, cast('now' as date)+1/8640, null, ".DB_TAR_TM_TASK_TRY_COUNT_LOWBALANCENOTIFY.", ?, ?)",
					array('text', 'text'));  //array('text', 'blob'));
				$result = $this->execute($pq, array($authnumber, $extparams));
			} else {
				//update exiting task
				$status = '';
				if ($task['tm_task_status'] >= TMS_STATUS_TECH_BLOCK)
					$status = ', tm_task_status='.TMS_STATUS_READY_FOR_DIAL;
				$q = "update tm_task_list set tm_task_params='$extparams' $status where tm_task_id=".$task['tm_task_id'];
				$pq = $this->prepare("update tm_task_list set tm_task_params=? $status where tm_task_id=?",
					array('text', 'integer'));  //array('blob', 'integer'));
				$result = $this->execute($pq, array($extparams, $task['tm_task_id']));
			}
		}

		if (PEAR::isError($result)) {
			$logger->log("updateLowBalanceNotifyTask($q):ERR=". getPEARErrorInfo($result));
			return null;
		} else {
			$logger->log("updateLowBalanceNotifyTask($q):".print_r($result, true));
			return is_array($result) && isset($result[0]['tm_task_id']) ? intval($result[0]['tm_task_id']) : true;
		}
	}

	/**
	 * get_tm_task_props
	 * @param $taskid
	 * @return array|null
	 */
	function get_tm_task_props($taskid) {
		//global $logger;

		$q = <<<EOS
			select tm_task_id,
				   tm_task_status,
				   tm_task_dial_num,
				   tm_task_try_count,
				   tm_task_auth_id,
				   cast(tm_task_params as varchar(4096)) as tm_task_params,
				   tm_task_result,
				   tm_task_last_time,
				   tm_task_dial_cause
			from tm_task_list
			where tm_task_id = $taskid
EOS;
		$res = $this->getQueryResultAsArray($q, true);
		//$logger->log("get_tm_task_props($taskid):" . print_r($res, true));
		return $res;
	}

	/**
	 * get_tm_task_props_by_authnumber
	 * @param $authnumber
	 * @param $tmprojectid
	 * @return array|null
	 */
	function get_tm_task_props_by_authnumber($authnumber, $tmprojectid) {
		global $logger;

		$q = <<<EOS
			select tm_task_id,
				   tm_task_status,
				   tm_task_dial_num,
				   tm_task_try_count,
				   tm_task_auth_id,
				   cast(tm_task_params as varchar(4096)) as tm_task_params,
				   tm_task_result,
				   tm_task_last_time,
				   tm_task_dial_cause
			from tm_task_list
			where tm_task_project_id=$tmprojectid and tm_task_auth_id='$authnumber'
EOS;
		$res = $this->getQueryResultAsArray($q, true);
		//$logger->log("get_tm_task_props_by_authnumber('$authnumber', $tmprojectid):" . print_r($res, true));
		return $res;
	}

	/**
	 * get_tm_task_stats
	 * @param $taskid
	 * @return array|null
	 */
	function get_tm_task_stats($taskid) {
		global $logger;

		$q = <<<EOS
			select first 10
				tm_task_start_time,
				tm_task_status,
				tm_task_dial_status
			from tm_task_statistic
			where tm_task_id=$taskid
			order by tm_task_start_time desc
EOS;
		$res = $this->getQueryResultAsArray($q);
		if (is_array($res))
			$res = array_reverse($res);
		//$logger->log("get_tm_task_stats($taskid):" . print_r($res, true));
		return $res;
	}
	

	function modifyAccountWeb($ofid, $uid, $uname, $upass, $umail, $user_name_by_id) {
		global $logger, $database;

		if ($ofid === null || !is_numeric($ofid))
			$ofid = 'null';
		if ($uid === null)
			$uid = 'null';

		$logger->log("modifyAccountWeb('$ofid', $uid, '$uname', '$upass', '$umail')", PEAR_LOG_DEBUG);

		if ($upass === null) {
			$pq = $this->prepare("select * from INT_UPDATE_ACCOUNTWEB($uid, ?, null, $ofid, ?, ?)",
									array('text','text','text'));
			$result = $this->execute($pq, array($uname, $umail, $user_name_by_id));
		} else {
			$pq = $this->prepare("select * from INT_UPDATE_ACCOUNTWEB($uid, ?, ?, $ofid, ?, ?)",
									array('text', 'text','text','text'));
			$result = $this->execute($pq, array($uname, $upass, $umail, $user_name_by_id));
		}

		if (PEAR::isError($result))
			$logger->log("modifyAccountWeb($ofid, $uid, '$uname', '***'):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			
		$result = (empty($result) || PEAR::isError($result)) ? false : $result[0]['out_res'];
		return $result;

	}

	// реквизиты ящика по основному номеру телефона офиса (один офис - один ящик)
	function getOfficeBox($number) {
		$q = <<<EOS
select 
    BOXCOMMONID,
    BOXPRIVATEID,
    EMITTERKCID,
    ACCOUNTID,
    BOXTYPEID,
    round(8000 * MAXSIZE / 1073741824 , 2) as MAXSIZE,
    --8000 * MAXSIZE / 1073741824 as MAXSIZE,
    DELAFTERDAYS,
    PWD,
    BOXNAME,
    USESIZE,
    coalesce(BOXLOOP,0) as BOXLOOP,
	case when MAXSIZE<USESIZE then 0 else 
	round(8000 * (coalesce(MAXSIZE,0)-coalesce(USESIZE,0)) / 1073741824 , 2) end as FREESIZE
from tbboxes where boxcommonid=$number;
EOS;
		//return $this->getQueryResultAsArray("select * from tbboxes where boxcommonid=$number;");
		return $this->getQueryResultAsArray($q);
	}
	
	function setOfficeBox($number,$maxsize,$boxloop){
		global $logger;

		if(empty($maxsize)) { $maxsize = 'null'; }
		if($boxloop!=1) $boxloop=0;
		//$logger->log("setOfficeBox($number,$maxsize,$boxloop)", PEAR_LOG_DEBUG);

		$q = "execute procedure int_setofficebox($number,$maxsize,$boxloop)";
		$result = $this->query($q);

		//if (PEAR::isError($result))
		//	$logger->log("setOfficeBox($number,$maxsize,$boxloop):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
	}

/*******************************************************************************************/

	function setOfficeSIP($number,$sipcount){
		global $logger;

		if(empty($sipcount)) { $sipcount = 'null'; }
/*		$q = <<<EOS
update tbaccountsupdate as u
set u.sipcount=$sipcount
where exists ( select 1 from tbboxes b
where u.emitterkcid=b.emitterkcid and u.accountid=b.accountid
and trim(b.boxcommonid) = trim('$number') )
EOS;*/
		$q = <<<EOS
update tbaccountsupdate as u
set u.sipcount=$sipcount
where exists ( select 1 from tbphones b
where u.emitterkcid=b.emitterkcid and u.accountid=b.accountid
and trim(b.numbera) = trim('$number') )
EOS;

		$logger->log("setOfficeSIP($number,$sipcount):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
		$logger->log("setOfficeSIP($q):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
		$result = $this->query($q);
		if (PEAR::isError($result))
		$logger->log("setOfficeSIP($number,$sipcount):ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
	}

	

	/* информация о счете по номеру офиса */
	function getOfficeUserInfo_Tar($num){
		global $logger;
		$num = !empty($num) ? $num : 'null';
//		$q = "select first 1 * from tbaccounts a inner join tbphones p on a.accountid=p.accountid and a.emitterkcid=p.emitterkcid where p.numbera=trim($num)";
		$q = <<<EOS
		select first 1 g.fio, a.accountid, a.emitterkcid, a.accountgroupid, u.sipcount from tbaccounts a inner join tbphones p on a.accountid=p.accountid and a.emitterkcid=p.emitterkcid 
		left join tbaccountsupdate u on a.accountid=u.accountid and a.emitterkcid=u.emitterkcid 
		left join tbgts g on g.emitterkcid=a.emitterkcid and g.accountid=a.accountid
		where p.numbera=trim($num);
EOS;
		$result = $this->getQueryResultAsArray($q);
		if (PEAR::isError($result)) {
			$logger->log("getOfficeUserInfo_Tar($num):". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return null;
		}
		//$res = $result[0]['fio'];

		return $result;
	}
	
	
			
	
	
	/* редактирование имени пользователя (компании) */
	function save_account_name($acc_name,$acc_accountid,$acc_emitterkcid){
		global $logger;
		//$q = "update tbaccounts set fio='$acc_name' where accountid=$acc_accountid and emitterkcid=$acc_emitterkcid";
		$q = "update tbgts set fio='$acc_name' where accountid=$acc_accountid and emitterkcid=$acc_emitterkcid";
		$result = $this->query($q);
		$logger->log("save_account_name($acc_name,$acc_accountid,$acc_emitterkcid):", PEAR_LOG_DEBUG);
		if (PEAR::isError($result)) {
			$logger->log("save_account_name($acc_name,$acc_accountid,$acc_emitterkcid):". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		} else { return true;}
	}
	
	//function getTarifInfo_Tar($acc_accountid,$acc_emitterkcid){
	function getTarifInfo_Tar($acc_accountgroupid){
	global $logger;
	$acc_accountgroupid = !empty($acc_accountgroupid) ? $acc_accountgroupid : 'null';
		$q =<<<EOS
select first 1
t.tarifpoliticname,
g.accountgroupid,
a.tarif,
c.currencyname,
t.tarifpoliticid 
from tbaccountgroups g inner join tbtarifpolitics t on g.tarifpoliticid=t.tarifpoliticid
left join tbabontarifpolitic p on t.tarifpoliticid=p.tarifpoliticid
left join tbabonplata a on p.abonid=a.abonid
left join tbcurrency c on a.currencyid=c.currencyid 
where g.accountgroupid = $acc_accountgroupid	
EOS;
		$result = $this->getQueryResultAsArray($q);
		if (PEAR::isError($result)) {
			$logger->log("getOfficeUserInfo_Tar($num):". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return null;
		}
		return $result;
	}


	/* редактирование размера ящика */
	function save_boxsize($txt_boxsize,$acc_accountid,$acc_emitterkcid){
		global $logger, $session;
		$ofnumber = $session->ofnumber;
		$q = "update tbboxes set maxsize='$txt_boxsize' where boxcommonid=$ofnumber";
		$result = $this->query($q);
		//$logger->log("save_account_name($txt_boxsize,$acc_accountid,$acc_emitterkcid):", PEAR_LOG_DEBUG);
		if (PEAR::isError($result)) {
			$logger->log("save_boxsize($txt_boxsize,$acc_accountid,$acc_emitterkcid):". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		} else { return true;}
	}

	/* редактирование максимального кол-ва sip счетов */
	function save_sipcount($txt_sipcount,$acc_accountid,$acc_emitterkcid){
		global $logger, $session;
		$q = "update tbaccountsupdate set sipcount='$txt_sipcount' where emitterkcid=$acc_emitterkcid and accountid=$acc_accountid";
		$result = $this->query($q);
		//$logger->log("save_account_name($txt_boxsize,$acc_accountid,$acc_emitterkcid):", PEAR_LOG_DEBUG);
		if (PEAR::isError($result)) {
			$logger->log("save_boxsize($txt_boxsize,$acc_accountid,$acc_emitterkcid):". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		} else { return true;}
	}
	

	/* разнообразная информация о ЛС */
	function getOfficeAccountInfo(){
		global $logger, $session;
		$accountid = $session->acc_accountid;
		$emitterkcid = $session->acc_emitterkcid;
		
		$q = "select * from int_GetAccountInfo($accountid,$emitterkcid)";
		$result = $this->getQueryResultAsArray($q);
		if (PEAR::isError($result)) {
			$logger->log("getAccountInfo($accountid,$emitterkcid):". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		} else { return $result;}
	}
	

	/* дерево тарифных политик */
	function getTarifPoliticList(){
		global $logger;
		
		$q = "select tarifpoliticid,coalesce(parentid,0) as parentid,tarifpoliticname from tbtarifpolitics";
		$result = $this->getQueryResultAsArray($q);
		if (PEAR::isError($result)) {
			$logger->log("getAccountInfo($accountid,$emitterkcid):". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		} else { return $result;}
	}

/*******************************************************************************************/

	/**
	 * updateWeekDays_Mask
	 */
	function updateWeekDays_Mask($daygroup_id, $weekdays_mask) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		$q = "select * from int_set_weekdaysmask($daygroup_id, $weekdays_mask)";
		$result = $this->connection->queryAll($q);
		if (PEAR::isError($result)) {
			$logger->log("$q.ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
		}
		//$logger->log("$q.result=".print_r($result, true), PEAR_LOG_DEBUG);
		return $result;
	}
	/**
	 * updateDayGroup
	 */
	function updateDayGroup($daygroup_id, $daygroup_name) {
		global $session, $logger;

		if ($session->isDemo())
			return null;

		$q = "select * from int_update_daygroup($daygroup_id, '$daygroup_name')";
		$result = $this->connection->queryAll($q);
		if (PEAR::isError($result)) {
			$logger->log("$q.ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
		}
		//$logger->log("$q", PEAR_LOG_DEBUG);

		//changelog
		/*$this->logOfficeChange($daygroup_id == -1 ? CL_OF_DAYGROUP_NEW :
							($daygroup_name == '' ? CL_OF_DAYGROUP_DELETE : CL_OF_DAYGROUP_CHANGE),
			$ofid, 0,
			($daygroup_id == -1 ? 'Добавление' : ($daygroup_name == '' ? 'Удаление' : 'Изменение')) .
				(" группы дней '$daygroup_name': ") .
				($result == null ? 'ошибка.' : 'успешно.')
			);*/

		return $result;
	}

	function getDayGroupsByOfficeID($ofid){
		return $this->getQueryResultAsArray("select * from tbdaygroups");
	}
	function getGraphListsByOfficeID($ofid){
		return $this->getQueryResultAsArray("select graphid,graphname from tbgraphs order by graphid");
	}
	function getDayGroups_htmlcode_ex($arr){
		$result = "";
		foreach ($arr as $row) {
			$result .= "daygrouplist['".addslashes(htmlspecialchars($row['daygroupname']))."'] = ".$row['daygroupid'].";";
		}
		return $result;
	}

	function updateGraphList($graph_id, $graph_name) {
		global $session, $logger;

		$graph_name = str_ireplace('\'', '"', $graph_name);
		
		//$graph_name = mb_convert_encoding($graph_name, "windows-1251");
//		$graph_name = iconv('utf-8','windows-1251//TRANSLIT',$graph_name); 

		$q = "select * from int_update_graphlist($graph_id, '$graph_name');";
		$result = $this->getQueryResultAsArray($q);
		if (PEAR::isError($result)) {
			$logger->log("updateGraphList($q).ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
		}
		return $result;
	}
	
	function updateGraphsMultiple($data) {
		global $session, $logger;

		$q = 'insert into tbgraphics (GRAPHID,DAYGROUPID,STARTTIME,ENDTIME) values (?, ?, ?, ?)';
		$result = $this->connection->prepare($q);
		if (PEAR::isError($result)) {
			$logger->log("updateGraphsMultiple($data).PREPARE.ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
			return $result;
		}

		/*$this->connection->extended->executeMultiple($result, $data);
		if (PEAR::isError($result)) {
			$logger->log("updateGraphsMultiple($data).EXEC.ERR=". getPEARErrorInfo($result), PEAR_LOG_ERR);
			$result = null;
			return $result;
		}
		$result->free();*/
		
		$this->connection->extended->executeMultiple($result, $data);
		

		$result = true;
		//$logger->log("updateGraphsMultiple($data).result=".print_r($result, true), PEAR_LOG_DEBUG);
		return $result;
	}

	
	
	function modifyUserAdmin($user_name, $user_role, $user_pass, $user_id, $user_mail, $rowtype, $user_name_old) {
		global $logger;

		switch ($rowtype){
			case 'insert':{
				$q = "insert into tbaccountweb (login, userpass, webtype_id, usermail, emitterkcid,accountid) values ('".$user_name."','".$user_pass."',4,'".$user_mail."',2,1);";
				$result = $this->connection->query($q);
				break;
			}
			case 'edit':{
				$q = "update tbaccountweb set login='".$user_name."', userpass='".$user_pass."', usermail='".$user_mail."' where login='$user_name_old'";
				$result = $this->connection->query($q);
				break;
			}
			case 'delete':{
				$q = "delete from tbaccountweb where login='$user_name'";
				$result = $this->connection->query($q);
				break;
			}
			default:
				$result = null;
		}

        if (PEAR::isError($result)) {
            $logger->log("modifyUserAdmin($user_name, $user_role, $user_pass, $user_id, $user_mail, $rowtype):". getPEARErrorInfo($result), PEAR_LOG_ERR);
            return null;
        }
		$result = (empty($result) || PEAR::isError($result)) ? false : $result;
		return $result;
	}






	/**
	 * get_stats_geographys_htmlcode
	 *
	 * @param $arr
	 * @param $selected_id
	 *
	 * @return string
	 */
	function get_stats_usersip_htmlcode($arr, $selected_id) {
		$result = '';
		foreach ($arr as $row) {
			$result .= '<option ';
			if ($selected_id == $row['name'])
				$result .= ' selected';
			$result .= '>' . addslashes(htmlspecialchars($row['name'])) . '</option>';
		}
		return $result;
	}
	/**
	 * get_stats_geographys
	 * @return array|null
	 */
	function get_stats_usersip() {
		global $session;

		//$number = DB_TAR_DEBUG ? DB_TAR_DEBUG_NUMBER : DB_TAR_FEDERAL_PREFIX . $session->ofnumber;
		//$starttm = date("d.m.Y", mktime(0, 0, 0, date("m"), date("d"), date("Y") - 2));
		//$starttm = date("d.m.Y", mktime(0, 0, 0, date("m") , date("d") - 1, date("Y") ));
		//$endtm = date("d.m.Y H:i");

		$q =<<<EOS
		select v4.name,v4.sip_id from
		(select * from tbaccountaccess where login = 'a') as v1
		inner join tbaccessgroup v2 on v1.accessgroup_id = v2.accessgroup_id
		inner join tbsipaccess v3 on v2.accessgroup_id = v3.accessgroup_id
		inner join tbsipconnections v4 on v3.sip_id = v4.sip_id
EOS;
		$result = $this->getQueryResultAsArray($q);
		//$result = array('geographyname' => 'erwgrrbhythnt');
		return $result;
	}

	function get_pbx_stats($flagsin = DB_TAR_SERVICETYPE_ALLTYPES,
					   $page = 1, $resultsperpage = 50,
					   $sortname = null, $sortorder = null,
					   $searchname = null, $searchquery = null,
					   &$totalcnt, &$totalvol, &$totalpay) {
		global $session, $logger;

		$filter = $session->filters[$flagsin];

		//$zz = $filter['sf_sipaccount'];
		//$logger->log("ZZZZZZZZZZ $zz", PEAR_LOG_DEBUG);
		
		//$number = DB_TAR_DEBUG ? DB_TAR_DEBUG_NUMBER : DB_TAR_FEDERAL_PREFIX . $session->ofnumber;
		$number = $filter['sf_sipaccount'];
		$startnum = 0;
		$endnum = -1;
		$starttm = $filter['sf_startdate'] . ' 00:00:00';
		$endtm = $filter['sf_enddate'] . ' 23:59:59';

		if (!$sortname)
			$sortname = 'creattime, sessionid';
		if (!$sortorder)
			$sortorder = 'desc';
		$sort = "order by $sortname $sortorder";

		if (!$page)
			$page = 1;
		if (!$resultsperpage)
			$resultsperpage = 50;
		$start = ($page - 1) * $resultsperpage;
		$limit = "first $resultsperpage skip $start";

		$addwhere = "";
		if ($searchquery) {
			$addwhere = "and $searchname like ";
			$addwhere .= (substr($searchquery, 0, 1) == '^') ?
					"'" . substr($searchquery, 1) . "%'" :
					"'%$searchquery%'";
		}

		$where = sprintf(" where (realvolume is null or realvolume >= %d and realvolume <= %d) ", 60 * $filter['sf_startvol'], 60 * $filter['sf_endvol']);
		if (in_array($flagsin, array(DB_TAR_SERVICETYPE_ALLTYPES, DB_TAR_SERVICETYPE_MTR, DB_TAR_SERVICETYPE_VOICE, DB_TAR_SERVICETYPE_FAX)) &&
			!empty($filter['sf_numbera']))
			$where .= sprintf(" and numbera like '%%'||'%s'||'%%' ", $filter['sf_numbera']);
		if (in_array($flagsin, array(DB_TAR_SERVICETYPE_ALLTYPES, DB_TAR_SERVICETYPE_MTR, DB_TAR_SERVICETYPE_VOICE_SENT, DB_TAR_SERVICETYPE_FAX_SENT)) &&
			!empty($filter['sf_numberb']))
			$where .= sprintf(" and numberb like '%%'||'%s'||'%%' ", $filter['sf_numberb']);
		if (in_array($flagsin, array(DB_TAR_SERVICETYPE_ALLTYPES, DB_TAR_SERVICETYPE_MTR))) {
			if (!empty($filter['sf_geo']))
				$where .= sprintf(" and geographyname like '%%'||'%s'||'%%' ", strtoupper($filter['sf_geo']));
			/*if ($filter['sf_calltype100'] != '1')
				$where .= " and calltype <> 100 ";
			if ($filter['sf_calltype101'] != '1')
				$where .= " and calltype <> 101 ";
			if ($filter['sf_calltype301'] != '1')
				$where .= " and calltype <> 301 ";*/
				// без входящих
				if ($filter['sf_calltype301'] <> '1')
					$where .= " and calltype <> 301 ";
				// без исходящиx
				if ($filter['sf_calltype201'] <> '1')
					$where .= " and calltype <> 201 ";
				// без пропущенных
				if ($filter['sf_calltype100'] <> '1')
					$where .= " and calltype <> 100 ";

		}

		$qcnt = "select count(creattime) as totalcnt, cast(sum(payablevolume)/60 as integer) as totalvol, sum(pay) as totalpay from INT_REPORTACCOUNTMTR_PBX('$number', null, '$starttm', '$endtm', $startnum, $endnum, $flagsin) $where $addwhere";
		$cntres = $this->getQueryResultAsArray($qcnt, true);
		$totalcnt = empty($cntres) ? 0 : $cntres['totalcnt'];
		if ($totalcnt > 0) {
			$totalvol = $cntres['totalvol'];
			$totalpay = $cntres['totalpay'];
			$q = "select $limit * from INT_REPORTACCOUNTMTR_PBX('$number', null, '$starttm', '$endtm', $startnum, $endnum, $flagsin) $where $addwhere $sort";
			//$logger->log("$q", PEAR_LOG_DEBUG);
			return $this->getQueryResultAsArray($q);
		} else
			return array();
	}


	function get_account_status($filter) {
	global $session, $logger;

		$date1 = trim($filter['sf_startdate']);
		$date2 = trim($filter['sf_enddate']);
		//$conditions = '';
		//$conditions .= empty($filter['f_account']) ? '' : ' lower(g.accountgts) like lower(\'%'.$filter['f_account'].'%\')';
		//$conditions .= empty($filter['f_number']) ? '' : (empty($conditions) ? '' : ' and ').'lower(p.numbera) like lower(\'%'.$filter['f_number'].'%\')';
		//$conditions = 'where '.(empty($conditions) ? ' 1<>1' : $conditions);
		
		//$q = "select distinct g.accountgts, g.fio, a.notify_address as address, a.emitterkcid, a.accountid from tbgts g inner join tbaccounts a on a.accountid=g.accountid left join tbphones p on a.accountid=p.accountid $conditions ";
		//$logger->log("  $q", PEAR_DEBUG);
		//return $this->getQueryResultAsArray($q);
		
		$q = "select accountgts,fio,operatedate,status,document,notify_address,emitterkcid,accountid from rpt_account_status('$date1','$date2')";
		return $this->getQueryResultAsArray($q);
	}

	
	
};

$database_tar = new TARDB;

?>
