<?php
/*
 * fileserver.php
 */
require_once(dirname(__FILE__) . '/constants.php');
require_once('Net/Socket.php');

define("FS_MAX_DATA_BLOCK", 8 * 1024);

/*Protocol constants*/
define("FS_PROT_CM_LOAD_MSG", 0);
define("FS_PROT_CM_SAVE_MSG", 1);
define("FS_PROT_CM_SAVE_DIR_MSG", 101);
define("FS_PROT_CM_LOAD_ERR", 17);
define("FS_PROT_CM_DELETE_MSG", 2);
define("FS_PROT_CM_SEND_MAIL", 4);

define("FS_PROT_MS_LOAD_MSG", 0);
define("FS_PROT_MS_SAVE_MSG", 1);
define("FS_PROT_MS_SAVE_DIR_MSG", 101);
define("FS_PROT_MS_ERROR_MSG", 17);

define("FS_PROT_DATA_ID_SIZE", 4);
define("FS_PROT_DATA_FILE_SIZE", 4);

/*Error constants*/
define("FS_ERROR_OK", 0);
define("FS_ERROR_FAIL_CONNECT", 1);
define("FS_ERROR_FAIL_SOCK_NAME", 2);
define("FS_ERROR_FAIL_SOCK_READ", 3);
define("FS_ERROR_FAIL_FILE_OPEN", 4);
define("FS_ERROR_FAIL_FILE_WRITE", 5);
define("FS_ERROR_FAIL_TPKT_FOUND", 6);
define("FS_ERROR_FAIL_WRONG_MSG", 7);
define("FS_ERROR_FAIL_LOAD_MSG", 8);
define("FS_ERROR_FAIL_SAVE_MSG", 9);
define("FS_ERROR_FAIL_FILE_READ", 10);
define("FS_ERROR_FAIL_FILE_PLACE", 11);

/*Timeout constants*/
define("FS_TO_WAIT_ERROR_SEC", 0);
define("FS_TO_WAIT_ERROR_USEC", 100000); //100 ms

class FS_MANIPULATOR {
	private $ControlSocket;
	private $DataSocket;
	private $LocalDataPort;
	private $LocalIPAddress;
	public $ErrorDesc;
	public $DebugMode = false;

	private function CreateConnection() {
		//Data socket connect
		$this->DataSocket = new Net_Socket();
		$this->DataSocket->connect(FILESERVER_HOST, FILESERVER_DATA_PORT, false, 30);
		if (PEAR::isError($this->DataSocket) || !$this->DataSocket->fp) {
			$this->ErrorDesc = FILESERVER_HOST . ":" . FILESERVER_DATA_PORT; //."!\n".print_r($this->DataSocket, true);
			return FS_ERROR_FAIL_CONNECT;
		}

		//Wait for connect will detected
		usleep(20000);

		//Control socket connect
		$this->ControlSocket = new Net_Socket();
		$this->ControlSocket->connect(FILESERVER_HOST, FILESERVER_ADMIN_PORT, false, 30);
		if (PEAR::isError($this->ControlSocket) || !$this->ControlSocket->fp) {
			$this->ErrorDesc = FILESERVER_HOST . ":" . FILESERVER_ADMIN_PORT; //."!\n".print_r($this->ControlSocket, true);
			return FS_ERROR_FAIL_CONNECT;
		}

		//Success
		$s_res = stream_socket_get_name($this->DataSocket->fp, false);
		$a_res = explode(":", $s_res);
		if (isset($a_res)) {
			$i = 0;
			foreach ($a_res as $val) {
				if ($i++ == 0)
					$this->LocalIPAddress = $val;
				else
					$this->LocalDataPort = (int)$val;
			}
			return FS_ERROR_OK;
		} else
		{
			$this->ErrorDesc = "Unsupported socket type";
			return FS_ERROR_FAIL_SOCK_NAME;
		}
	}

	private function DestroyConnection() {
		$this->DataSocket->disconnect();
		$this->ControlSocket->disconnect();
	}

	private function ReceiveDataStream($file_name) {
		$res = $this->DataSocket->read(FS_PROT_DATA_ID_SIZE);
		if (PEAR::isError($this->DataSocket)) {
			$this->ErrorDesc = FILESERVER_HOST . ":" . FILESERVER_ADMIN_PORT; //."!\n".print_r($this->DataSocket, true);
			return FS_ERROR_FAIL_SOCK_READ;
		} else {
			if ($res == null) {
				$this->ErrorDesc = "found EOF from data socket in ID place";
				return FS_ERROR_FAIL_SOCK_READ;
			}
		}

		$res = $this->DataSocket->read(FS_PROT_DATA_FILE_SIZE);
		if (PEAR::isError($this->DataSocket)) {
			$this->ErrorDesc = FILESERVER_HOST . ":" . FILESERVER_ADMIN_PORT; //."!\n".print_r($this->DataSocket, true);
			return FS_ERROR_FAIL_SOCK_READ;
		} else {
			if ($res == null) {
				$this->ErrorDesc = "found EOF from data socket in FILE SIZE place";
				return FS_ERROR_FAIL_SOCK_READ;
			}
		}

		$f_dsc = fopen($file_name, 'wb');
		if ($f_dsc === false) {
			$this->ErrorDesc = "cannot create file [$file_name]";
			return FS_ERROR_FAIL_FILE_OPEN;
		}

		$i_size = ord(substr($res, 0, 1)) + (ord(substr($res, 1, 1)) << 8) + (ord(substr($res, 2, 1)) << 16) + (ord(substr($res, 3, 1)) << 24);
		$i_blocks = (int)($i_size / FS_MAX_DATA_BLOCK);
		$i_end = $i_size % FS_MAX_DATA_BLOCK;
		if ($this->DebugMode)
			print("Receive file [size=$i_size,blocks=$i_blocks,end=$i_end]\n");

		$i = 0;
		while ($i < $i_size) {
			//read data from socket
			if ($i_size > FS_MAX_DATA_BLOCK) {
				if (($i_size - $i) > FS_MAX_DATA_BLOCK)
					$res = $this->DataSocket->read(FS_MAX_DATA_BLOCK);
				else
					$res = $this->DataSocket->read($i_size - $i);
			} else
				$res = $this->DataSocket->read($i_size);

			$i += strlen($res);
			if (PEAR::isError($this->DataSocket)) {
				$this->ErrorDesc = FILESERVER_HOST . ":" . FILESERVER_ADMIN_PORT; //."!\n".print_r($this->DataSocket, true);
				fclose($f_dsc);
				return FS_ERROR_FAIL_SOCK_READ;
			} else {
				if ($res == null) {
					$this->ErrorDesc = "found EOF from data socket in Data place";
					fclose($f_dsc);
					return FS_ERROR_FAIL_SOCK_READ;
				}
			}

			//put data to file
			$i_written = fwrite($f_dsc, $res);
			if ($i_written == false) {
				$this->ErrorDesc = "error while writing data to file [$file_name]";
				fclose($f_dsc);
				return FS_ERROR_FAIL_FILE_WRITE;
			} else {
				if ($this->DebugMode)
					print("fwrite OK [wr=" . $i_written . "] \n");
			}
		}

		if (fclose($f_dsc)) {
			if ($this->DebugMode)
				print("fclose OK [" . filesize($file_name) . "] \n");
		} else {
			if ($this->DebugMode)
				print("fclose FAIL [" . filesize($file_name) . "] \n");
		}

		return FS_ERROR_OK;
	}

	private function SendDataStream($file_name, $file_id) {
		$f_size = filesize($file_name);
		//print("File size=$f_size, File ID=$file_id\n");
		if ($f_size <= 0) {
			$this->ErrorDesc = "error while reading file size [$file_name]";
			return FS_ERROR_FAIL_FILE_READ;
		}

		$f_dsc = fopen($file_name, 'rb');
		if ($f_dsc === false) {
			$this->ErrorDesc = "cannot read file [$file_name]";
			return FS_ERROR_FAIL_FILE_OPEN;
		}

		//send data
		$i_blocks = (int)($f_size / FS_MAX_DATA_BLOCK);
		$i_end = $f_size % FS_MAX_DATA_BLOCK;
		for ($i = 0; $i <= $i_blocks; $i++) {
			$i_block_size = FS_MAX_DATA_BLOCK;
			if ($i == $i_blocks) $i_block_size = $i_end;
			//read data from file
			$s_packet = fread($f_dsc, $i_block_size);
			if ($s_packet === false) {
				$this->ErrorDesc = "cannot read file [$file_name]";
				fclose($f_dsc);
				return FS_ERROR_FAIL_FILE_READ;
			}
			if ($i == 0) {
				$s_packet = chr($file_id) . chr($file_id >> 8) .
						chr($file_id >> 16) . chr($file_id >> 24) .
						chr($f_size) . chr($f_size >> 8) .
						chr($f_size >> 16) . chr($f_size >> 24) . $s_packet;
			}

			if ($this->DebugMode)
				print("Send file block [size=$i_block_size,i=$i] \n");

			$this->DataSocket->write($s_packet);

			if (($err_code = $this->CheckForError())) {
				fclose($f_dsc);
				return $err_code;
			}
		}

		fclose($f_dsc);
		return $this->CheckForError(FS_TO_WAIT_ERROR_SEC, FS_TO_WAIT_ERROR_USEC);
	}

	private function CheckForError($sec_time_out = 0, $u_sec_time_out = 0) {
		$read = array($this->ControlSocket->fp);

		if (false === ($num_changed_streams = stream_select($read, $write = NULL, $except = NULL, $sec_time_out, $u_sec_time_out))) {
			/* Error handling */
			$this->ErrorDesc = "cannot stream_select";
			return FS_ERROR_FAIL_SOCK_READ;
		} elseif ($num_changed_streams > 0) {
			/* At least on one of the streams something interesting happened */
			if ($this->DebugMode)
				print("Receive some MS from FileServer while send file!\n");
			$msg = '';
			if (($res = $this->ReceiveControlAnswer($msg))) {
				return $res;
			} else {
				$msg_code = ord(substr($msg, 0, 1));
				if ($msg_code == FS_PROT_MS_ERROR_MSG) {
					$msg_err = (strlen($msg) >= 20) ? ord(substr($msg, 19, 1)) : 'Unknown';
					$this->ErrorDesc = "error $msg_err";
					return FS_ERROR_FAIL_FILE_PLACE;
				} else {
					if ($this->DebugMode)
						print("Ignore unwaitable message!\n");
					return FS_ERROR_OK; //Ignore
				}
			}
		}
		return FS_ERROR_OK; //All right!!!
	}

	private function SendCommand($cmd, &$msg) {
		$len_o_send = strlen($cmd) + 4;
		//$to_send=chr(3).chr(0).chr($len_o_send).chr($len_o_send>>8).$cmd;
		$to_send = chr(3) . chr(0) . chr($len_o_send >> 8) . chr($len_o_send) . $cmd;

		if ($this->DebugMode) {
			print("Send CM to FileServer:");
			for ($i = 0; $i < strlen($to_send); $i++)
				print(" " . ord(substr($to_send, $i, 1)));
			print("\n");
		}

		$this->ControlSocket->write($to_send);

		return $this->ReceiveControlAnswer($msg);
	}

	private function ReceiveControlAnswer(&$msg) {
		$res = $this->ControlSocket->read(4);
		if (PEAR::isError($this->ControlSocket)) {
			$this->ErrorDesc = FILESERVER_HOST . ":" . FILESERVER_ADMIN_PORT; //."!\n".print_r($this->ControlSocket, true);
			return FS_ERROR_FAIL_SOCK_READ;
		} else {
			if ($res == null) {
				$this->ErrorDesc = "found EOF from control socket in Header place";
				return FS_ERROR_FAIL_SOCK_READ;
			}
		}
		$msg_len = (ord(substr($res, 2, 1)) << 8) + (ord(substr($res, 3, 1))) - 4;

		if ($this->DebugMode) {
			print("Receive MS from FileServer:");
			for ($i = 0; $i < strlen($res); $i++)
				print(" " . ord(substr($res, $i, 1)));
		}

		if ($msg_len > 0) {
			$msg = $this->ControlSocket->read($msg_len);
			if (PEAR::isError($this->ControlSocket)) {
				$this->ErrorDesc = FILESERVER_HOST . ":" . FILESERVER_ADMIN_PORT; //."!\n".print_r($this->ControlSocket, true);
				return FS_ERROR_FAIL_SOCK_READ;
			} else {
				if ($res == null) {
					$this->ErrorDesc = "found EOF from control socket in Data place";
					return FS_ERROR_FAIL_SOCK_READ;
				}
			}
			if ($this->DebugMode) {
				for ($i = 0; $i < strlen($msg); $i++)
					print(" " . ord(substr($msg, $i, 1)));
				print("\n");
			}
			return FS_ERROR_OK;
		} else {
			$this->ErrorDesc = "wrong TPKT header found";
			return FS_ERROR_FAIL_TPKT_FOUND;
		}
	}

	private function SendLoadMessage($file_id) {
		$cmLoadMSG = chr(FS_PROT_CM_LOAD_MSG);
		//make packet
		$s_tmp = $this->LocalIPAddress;
		while (strlen($s_tmp) < 16)
			$s_tmp .= chr(0);
		$cmLoadMSG .= $s_tmp . chr($this->LocalDataPort) . chr($this->LocalDataPort >> 8);
		$cmLoadMSG .= chr($file_id) . chr($file_id >> 8) . chr($file_id >> 16) . chr($file_id >> 24);
		//send command
		$msg = '';
		$i_res = $this->SendCommand($cmLoadMSG, $msg);
		//operate answer
		if ($i_res == FS_ERROR_OK) {
			if (strlen($msg) >= 20) {
				$msg_code = ord(substr($msg, 0, 1));
				if ($msg_code == FS_PROT_MS_LOAD_MSG) {
					$msg_err = ord(substr($msg, 19, 1));
					if ($msg_err !== 0) {
						$this->ErrorDesc = "msLoadMessage Error <$msg_err>";
						return FS_ERROR_FAIL_LOAD_MSG;
					}
					return FS_ERROR_OK;
				} else {
					$this->ErrorDesc = "[on waiting msLoadMessage but found <$msg_code>]";
					return FS_ERROR_FAIL_WRONG_MSG;
				}
			} else {
				$this->ErrorDesc = "[on waiting msLoadMessage but found some MSG with " . strlen($msg) . " length]";
				return FS_ERROR_FAIL_WRONG_MSG;
			}
		} else
			return $i_res;
	}

	private function SendSaveMessage(&$file_id, $off_converter = false) {
		$cmSaveMSG = chr(FS_PROT_CM_SAVE_MSG);
		if ($off_converter) $cmSaveMSG = chr(FS_PROT_CM_SAVE_DIR_MSG);
		//make packet
		$s_tmp = $this->LocalIPAddress;
		while (strlen($s_tmp) < 16)
			$s_tmp .= chr(0);
		$cmSaveMSG .= $s_tmp . chr($this->LocalDataPort) . chr($this->LocalDataPort >> 8);
		if (isset($file_id))
			$cmSaveMSG .= chr($file_id) . chr($file_id >> 8) . chr($file_id >> 16) . chr($file_id >> 24);
		//send command
		$msg = '';
		$i_res = $this->SendCommand($cmSaveMSG, $msg);
		//operate answer
		if ($i_res == FS_ERROR_OK) {
			if (strlen($msg) >= 20) {
				$msg_code = ord(substr($msg, 0, 1));
				if (($msg_code == FS_PROT_MS_SAVE_MSG) || ($msg_code == FS_PROT_MS_SAVE_DIR_MSG)) {
					$msg_err = ord(substr($msg, 19, 1));
					if ($msg_err !== 0) {
						$this->ErrorDesc = "msSaveMessage Error <$msg_err>";
						return FS_ERROR_FAIL_SAVE_MSG;
					}
					if (strlen($msg) < 24) {
						$this->ErrorDesc = "msSaveMessage Error: cannot found file ID in message";
						return FS_ERROR_FAIL_SAVE_MSG;
					}
					$file_id = ord(substr($msg, 20, 1)) + (ord(substr($msg, 21, 1)) << 8) + (ord(substr($msg, 22, 1)) << 16) + (ord(substr($msg, 23, 1)) << 24);
					return FS_ERROR_OK;
				} else {
					$this->ErrorDesc = "[on waiting msSaveMessage but found <$msg_code>]";
					return FS_ERROR_FAIL_WRONG_MSG;
				}
			} else {
				$this->ErrorDesc = "[on waiting msSaveMessage but found some MSG with " . strlen($msg) . " length]";
				return FS_ERROR_FAIL_WRONG_MSG;
			}
		} else
			return $i_res;
	}

	private function SendDeleteMessage($file_id) {
		$cmDeleteMSG = chr(FS_PROT_CM_DELETE_MSG);
		//make packet
		$cmDeleteMSG .= chr($file_id) . chr($file_id >> 8) . chr($file_id >> 16) . chr($file_id >> 24);
		//send command
		$msg = '';
		$i_res = $this->SendCommand($cmDeleteMSG, $msg);
		//operate answer
		if ($i_res == FS_ERROR_OK) {
			//ToDo - Make processing
			return FS_ERROR_OK;
		} else
			return $i_res;
	}

	/*
		Interface functions
	*/
	public function PlaceFile($file_name, &$file_id, $off_converter = false, $remove_source_file = false) {
		$err = $this->CreateConnection();
		if ($err == FS_ERROR_OK) {
			$err = $this->SendSaveMessage($file_id, $off_converter);
			if ($err == FS_ERROR_OK) {
				$err = $this->SendDataStream($file_name, $file_id);
				if ($err !== FS_ERROR_OK)
					$this->DeleteFile($file_id, false);
			}
			$this->DestroyConnection();
			if ($remove_source_file == true)
				unlink($file_name);
			return $err;
		}
		if ($remove_source_file == true)
			unlink($file_name);
		return $err;
	}

	public function DeleteFile($file_id, $in_new_connection = true) {
		$err = FS_ERROR_OK;
		if ($in_new_connection) $err = $this->CreateConnection();
		if ($err == FS_ERROR_OK) {
			$err = $this->SendDeleteMessage($file_id);
			if ($in_new_connection) $this->DestroyConnection();
		}
		return $err;
	}

	public function ReceiveFile($file_id, $file_name) {
		$err = $this->CreateConnection();
		if ($err == FS_ERROR_OK) {
			$err = $this->SendLoadMessage($file_id);
			if ($err == FS_ERROR_OK)
				$err = $this->ReceiveDataStream($file_name);
			$this->DestroyConnection();
			return $err;
		}
		return $err;
	}

	public function ErrorDescription($error) {
		if (!isset($error))
			return "Error code is not defined";
		switch ($error) {
			case FS_ERROR_OK:
				return "Operation completed successfully";
				break;
			case FS_ERROR_FAIL_CONNECT:
				return "Failed: connect operation [" . $this->ErrorDesc . "]";
				break;
			case FS_ERROR_FAIL_SOCK_NAME:
				return "Failed: get socket name [" . $this->ErrorDesc . "]";
				break;
			case FS_ERROR_FAIL_SOCK_READ:
				return "Failed: read socket data [" . $this->ErrorDesc . "]";
				break;
			case FS_ERROR_FAIL_FILE_OPEN:
				return "Failed: open file [" . $this->ErrorDesc . "]";
				break;
			case FS_ERROR_FAIL_FILE_WRITE:
				return "Failed: write data to file [" . $this->ErrorDesc . "]";
				break;
			case FS_ERROR_FAIL_TPKT_FOUND:
				return "Wrong TPKT [" . $this->ErrorDesc . "]";
				break;
			case FS_ERROR_FAIL_WRONG_MSG:
				return "Wrong message [" . $this->ErrorDesc . "]";
				break;
			case FS_ERROR_FAIL_LOAD_MSG:
				return "Failed: load message [" . $this->ErrorDesc . "]";
				break;
			case FS_ERROR_FAIL_SAVE_MSG:
				return "Failed: save message [" . $this->ErrorDesc . "]";
				break;
			case FS_ERROR_FAIL_FILE_PLACE:
				return "Failed: place file [" . $this->ErrorDesc . "]";
				break;
			default:
				return "Unknown error [" . $this->ErrorDesc . "]";
		}
	}
}

$fileserver_control = new FS_MANIPULATOR;

?>
