<?php
require_once 'Image/GraphViz.php';
require_once(dirname(__FILE__).'/include/session.php');
require_once(dirname(__FILE__).'/include/database_dynmenu.php');

global $session, $database_dynmenu, $logger;

define("GROUP_NODES_CONNECTED", 'cnodes');
define("GROUP_NODES_SPARE", 'snodes');
//define("WRITE_RIGHT_STATE", $session->checkUserRight('adm_menuedit.php','W'));

if( ($session->checkUserRight('adm_menuedit.php','W')) or ($session->checkUserRight('menus.php','W')) )
define("WRITE_RIGHT_STATE", true);

$menu_start_item_id = -1;
$graph = null;

function is_StartMenuItem($id) {
	global $menu_start_item_id;
	return $id == $menu_start_item_id;
}

function checkGraphNodeExist(&$nodescache, $graphobj, $item_id, $item_type, $item_name, $is_connected = true) {
	if (is_numeric($item_id) && !array_key_exists($item_id, $nodescache)) {
		//$logger->log("checkGraphNodeExist >> ADDED\r\n");

		$shape = ($item_type > 2 || $item_type == 0) ? 'house' : ($item_type == 2 ? 'octagon' : 'egg');  //egg
		//$shape = $item_type == 2 ? 'octagon' : 'egg';
		$labelsuffix = $shape == 'egg' ? '' : "\n ";
		$style = is_StartMenuItem($item_id) ? 'filled' : 'filled';
		$fillcolor = is_StartMenuItem($item_id) ? 'lightgrey' : 'white';
		//$rank = $is_connected ? 'min' : 'max';
		$group = $is_connected ? GROUP_NODES_CONNECTED : GROUP_NODES_SPARE;
		//'style' => 'filled',
		//'fillcolor' => 'lightgrey',

		if( WRITE_RIGHT_STATE ) { $urltype = "javascript:parent.showMenuItemProps('$item_id');"; }
		else { $urltype = "javascript:alert('".MSG_READONLY."');"; }
		
		$graphobj->addNode(
			$item_id,
			array(
//				'URL'   => "javascript:parent.showMenuItemProps('$item_id');",//'?'.$item_id,
				'URL'   => $urltype,
				'label' => $item_id."\n".$item_name.$labelsuffix,
				'tooltip' => $item_name,
				'fontsize' => '14',
				'fontname' => 'Arial',
				'style' => $style,
				'fillcolor' => $fillcolor,
				//'rank' => $rank,
				'shape' => $shape
			),
			$group
		);
		$nodescache[$item_id] = $item_name;

		return false;
	}
	return true;
}

function initGraph($menuid) {
	global $database_dynmenu, $logger, $menu_start_item_id;

	if (!$database_dynmenu->db_ok) {
		$logger->log("initGraph(menuid=$menuid) -> ERROR: database not connected");
		return null;
	}

	$q = "select * from menu_get_start_item($menuid)";
	$result = $database_dynmenu->getQueryResultAsArray($q);
	if (PEAR::isError($result)) {
		$logger->log("initGraph(menuid=$menuid):ERR=". getPEARErrorInfo($result));
		return $result;
	} else {
		$menu_start_item_id = $result[0]['item_start_id'];
		$logger->log("initGraph(menuid=$menuid) -> StartItemId=$menu_start_item_id");
	}

	$q = "select * from menu_get_items_as_table($menuid)";
	$result = $database_dynmenu->getQueryResultAsArray($q);
	if (PEAR::isError($result)) {
		$logger->log("initGraph(menuid=$menuid):ERR=". getPEARErrorInfo($result));
		return $result;
	} else {
		$graph = new Image_GraphViz(true,
			array(
				//'bgcolor' => 'transparent',
				'bgcolor' => 'white',
				//'concentrate' => 'true',
				//'dpi' => '96',
				//'overlap' => 'false',
				'ratio' => 'fill',
				'rankdir' => 'TB',
				'clusterrank' => 'global',//local
				'fontnames' => 'svg',
				'fontsize' => '12',
				'fontname' => 'Arial',
				'charset' => 'UTF-8'
			),
			'', false, false);
		$graph->addCluster(GROUP_NODES_CONNECTED, '', array('color' => 'white'));
		$graph->addCluster(GROUP_NODES_SPARE, 'Свободные элементы', array('color' => '#ccc', 'style' => 'dotted'));
		$nodes = array();
		if (!empty($result))
		foreach ($result as $row) {
			//$logger->log("initGraph:row=\n".print_r($row, true));
			checkGraphNodeExist($nodes, $graph, $row['idsrc'], $row['idtypesrc'], $row['namesrc'], is_numeric($row['iddst']));
			checkGraphNodeExist($nodes, $graph, $row['iddst'], $row['idtypedst'], $row['namedst']);
			if ($row['iddst']) {
				$keyinfo = (strlen($row['itemkey']) > 0) ? " ({$row['itemkey']})" : '';
				$graph->addEdge(
					array(
						$row['idsrc'] => $row['iddst']
					),
					array(
						'URL'   => "javascript:parent.showMenuConnProps('". $row['idconn'].'|'.base64_encode($row['itemkey']) .'|'.$row['idsrc'].'|'.$row['iddst']. "');",
						'label' => wordwrap($row['keydesc'], 10) . $keyinfo,
						'labeltooltip' => $row['idsrc'] . ' > ' . $row['keydesc'] . $keyinfo . ' > ' . $row['iddst'],
						'tooltip' => $row['idsrc'] . ' > ' . $row['keydesc'] . $keyinfo . ' > ' . $row['iddst'],
						'fontsize' => '11',
						//'fontname' => 'Arial',
						'color' => 'darkorchid3',
						'labelfontsize' => '11',
						'labelfontname' => 'Arial',
						'labelfontcolor' => 'red',
						'style' => 'dashed'
					)
				);
			}
		}
		return $graph;
	}
}

function printGraphImage_NoMenu() {
	if (is_readable(dirname(__FILE__).'/'.MENU_FILE_NOMENU)) {
		header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		header('Cache-Control: no-store, no-cache, must-revalidate');
		header('Cache-Control: post-check=0, pre-check=0', FALSE);
		header('Pragma: no-cache');
		header('Content-Type: image/svg+xml');

		$res = false;
		ob_start();
		if (($fp = fopen(dirname(__FILE__).'/'.MENU_FILE_NOMENU, 'r'))) {
			while (!feof($fp)) {
				$buff = fread($fp, 4096);
				print $buff;
			}
			fclose($fp);
			$res = true;
		};
		ob_end_flush();
		flush();
		return $res;
	} else {
		header('Content-Type: image/html');
		echo "<img alt=\"\" src=\"images/icon_error2.gif\">";
		return false;
	}
}

function printGraphImage($graph) {
	if (is_a($graph, 'Image_GraphViz')) {
		header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		header('Cache-Control: no-store, no-cache, must-revalidate');
		header('Cache-Control: post-check=0, pre-check=0', FALSE);
		header('Pragma: no-cache');
		header('Content-Type: image/svg+xml');

		session_write_close();
		$graph->image('svg', 'dot');
		return true;
	} else {
		printGraphImage_NoMenu();
		/*header('Content-Type: image/html');
		echo "<img src=\"images/icon_error2.gif\">";*/
		return false;
	}
}

function getGraphImageData() {
	global $graph;

	session_write_close();
	if (is_a($graph, 'Image_GraphViz'))
		return $graph->fetch('png', 'dot');
	else
		return null;
}

function getGraphImageMapData() {
	global $graph;

	session_write_close();
	if (is_a($graph, 'Image_GraphViz'))
		return $graph->fetch('cmapx', 'dot');
	else
		return null;
}

//$logger->log($_SERVER['SCRIPT_NAME']);
//$logger->log('SERVER:'.print_r($_SERVER, true));
//$logger->log('REQUEST:'.print_r($_REQUEST, true));
//$logger->log('GET:'.print_r($_GET, true));
if (!empty($_GET) /*work only with real params. this also skips calling from 'include'*/) {
	if (isset($_REQUEST['nomenu'])) {
		printGraphImage_NoMenu();
	} else {
		//$menuid = ($session->logged_in && is_numeric($session->ofmenuid)) ? intval($session->ofmenuid) : (isset($_COOKIE['office_menuid']) ? intval($_COOKIE['office_menuid']) : -1);
		if (isset($_REQUEST['ivr_edit']))
			$menuid = isset($_COOKIE['office_menuid']) ? intval($_COOKIE['office_menuid']) : -1;
		else
			$menuid = ($session->logged_in && is_numeric($session->ofmenuid)) ? intval($session->ofmenuid) : -1;
		$menu_start_item_id = -1;
		if ($menuid >= 0) {
			$graph = initGraph($menuid);
			if ($graph && basename($_SERVER['SCRIPT_NAME']) == basename(__FILE__)) {
				//$logger->log($graph->parse(), PEAR_LOG_DEBUG);
				printGraphImage($graph);
			} else
				printGraphImage_NoMenu();
		}
	}
}
//getGraphImageData();
//$logger->log($graph->fetch('xdot', 'dot'));
?>
