<?php
require_once 'System.php';
require_once(dirname(__FILE__).'/include/database_dynmenu.php');
require_once(dirname(__FILE__).'/include/session.php');
require_once(dirname(__FILE__).'/include/functions.php');

global $database_dynmenu, $logger, $session, $upload_errors;

$jsonresErr = array('error' => 'Внутренняя ошибка!');
$jsonresErr['identifier'] = 'menu_id';
$jsonresErr['label'] = 'menu_name';
$jsonresErr['items'] = array();


function getMenusList($clean_menu_name = false, $sortname = null, $sortorder = null) {
	global $database_dynmenu, /*$jsonres,*/ $logger;

	if (!$sortname)
		$sortname = 'menu_name';
	if (!$sortorder)
		$sortorder = 'desc';
	$sort = "order by $sortname $sortorder";

	$name_prefix = $clean_menu_name ? '' : 'menu_id||\': \'||';
	$q = "select menu_id, {$name_prefix}menu_name as menu_name, menu_start_item from menu_get_menus_list $sort";
	return $database_dynmenu->getQueryResultAsArray($q);
}

function cloneMenu($in_menuid, $in_menuname) {
	global $database_dynmenu, $logger;

	if ($in_menuid == null || $in_menuname == null)
		return null;

	$q = "select * from global_menu_duplicate($in_menuid, '$in_menuname')";
	$result = $database_dynmenu->getQueryResultAsArray($q, true);
	if ($result)
		$logger->log("cloneMenu($in_menuid, '$in_menuname'):OK=".print_r($result, true));
	return $result;
}

function renameMenu($in_menuid, $in_menuname) {
	global $database_dynmenu, $logger;

	if ($in_menuid == null || $in_menuname == null)
		return null;

	$pq = $database_dynmenu->prepare("update tb_menus set menu_name=? where menu_id=?", array('text', 'integer'), MDB2_PREPARE_MANIP);
	$result = $database_dynmenu->execute($pq, array($in_menuname, $in_menuid));

	if (PEAR::isError($result) || !is_numeric($result)) {
		$logger->log("renameMenu($in_menuid, '$in_menuname'):ERR=". getPEARErrorInfo($result));
		return null;
	} else {
		$logger->log("renameMenu($in_menuid, '$in_menuname'):OK=".print_r($result, true));
		return intval($result);
	}
}

function insertMenu($in_menuid, $in_menuid_dst) {
	global $database_dynmenu, $logger;

	if ($in_menuid == null || $in_menuid_dst == null)
		return null;

	$q = "select * from insert_menu($in_menuid, $in_menuid_dst)";
	$result = $database_dynmenu->getQueryResultAsArray($q, true);
	if ($result)
		$logger->log("insertMenu($in_menuid, $in_menuid_dst):OK=".print_r($result, true));
	return $result;
}

function newMenu($in_menuname) {
	global $database_dynmenu, $logger;

	if ($in_menuname == null)
		return null;

	$q = "select * from global_menu_create('$in_menuname', 1)";
	$result = $database_dynmenu->getQueryResultAsArray($q, true);
	if (PEAR::isError($result)) {
		$logger->log("newMenu('$in_menuname'):ERR=". getPEARErrorInfo($result));
		return null;
	} else {
		$logger->log("newMenu('$in_menuname'):OK=".print_r($result, true));
		return $result;
	}
}

function deleteMenu($in_menuid) {
	global $database_dynmenu, $logger;

	if ($in_menuid == null)
		return null;

	$q = "execute procedure global_menu_delete($in_menuid)";
	$result = $database_dynmenu->query($q);
	if (PEAR::isError($result)) {
		$logger->log("deleteMenu($in_menuid):ERR=". getPEARErrorInfo($result));
		return null;
	} else {
		$logger->log("deleteMenu($in_menuid):OK");
		return Array('out_menu_id' => $in_menuid);
	}
}

function exportMenu($in_menuid, $in_menuname) {
	global $database_dynmenu, $logger;

	if ($in_menuid == null || $in_menuname == null)
		return null;

	$outfile = System::mktemp('blob_');
	if (($fp = fopen($outfile, 'wb'))) {
		$result = $database_dynmenu->query("select info from global_menu_export($in_menuid, '$in_menuname')", array('info'=>'blob'));
		if (PEAR::isError($result)) {
			$logger->log("exportMenu($in_menuid, '$in_menuname'):ERR=". getPEARErrorInfo($result));
			fwrite($fp, 'Ошибка экспорта');
		} else {
			$row = $result->fetchRow();
			$blob = $row['info'];
			if (!PEAR::isError($blob) && is_resource($blob)) {
				while (!feof($blob))
					fwrite($fp, fread($blob, 8192));
				$database_dynmenu->destroyLOB($blob);
			}
			$result->free();
			$logger->log("exportMenu($in_menuid, '$in_menuname'):OK");
		}
		fclose($fp);
		return $outfile;
	}
	return null;
}

//$logger->log("POST(".basename($_SERVER['SCRIPT_NAME']).":\n".print_r($_POST, true));

header("Content-Type: text/plain");
if ($database_dynmenu->db_ok) {
	$mode = isset($_POST['menus_operation']) ? $_POST['menus_operation'] : 'list';
	$menuid = isset($_POST['office_menuid']) ? $_POST['office_menuid'] : null;
	$menuname = isset($_POST['office_menuname']) ? $_POST['office_menuname'] : null;
	$menuid2 = isset($_POST['office_menuid2']) ? $_POST['office_menuid2'] : null;

	switch ($mode) {
		case 'list':
			$json = getMenusList();
			if (is_array($json)) {
				$jsonres = Array();
				$jsonres['identifier'] = 'menu_id';
				$jsonres['label'] = 'menu_name';
				$jsonres['ok'] = true;
				$jsonres['items'] = $json;
				//$logger->log('Menus list:' . print_r(json_encode_safe($jsonres), true));
				echo json_encode_safe($jsonres);
			} else
				echo json_encode_safe($jsonresErr);
			break;
		case 'gridlist':
			$json = getMenusList(true, $_POST['sortname'], $_POST['sortorder']);
			if (is_array($json)) {
				$jsonres = array(
					'page' => 1,
					'total' => count($json),
					'rows' => array()
				);
				foreach ($json as $row)
					$jsonres['rows'][] = array(
						'id' => $row['menu_id'],
						'cell' => array($row['menu_id'], $row['menu_name'])
					);
				echo json_encode_safe($jsonres);
			} else
				echo json_encode_safe($jsonresErr);
			break;
		case 'rename':
			$menuname_old = $database_dynmenu->getMenuInfo($menuid);
			$res = renameMenu($menuid, $menuname);
			if (!empty($res)) {
				$jsonres = Array();
				$jsonres['ok'] = true;
				$jsonres['out_menuid'] = $menuid;
				//$logger->log('Rename menu:' . print_r(json_encode_safe($jsonres), true));
				echo json_encode_safe($jsonres);

				//changelog
				$database_dynmenu->logMenuChange(CL_MENUS_MENURENAME, $menuid, 0,
					"Сценарий '$menuname_old' (#$menuid) переименован в '$menuname'.");
			} else
				echo json_encode_safe($jsonresErr);
			break;
		case 'clone':
			$json = cloneMenu($menuid, $menuname);
			if (is_array($json) && isset($json['out_menu_id'])) {
				$jsonres = Array();
				$jsonres['ok'] = true;
				$jsonres['out_menuid'] = $json['out_menu_id'];
				//$logger->log('Clone menu:' . print_r(json_encode_safe($jsonres), true));
				echo json_encode_safe($jsonres);

				//changelog
				$database_dynmenu->logMenuChange(CL_MENUS_MENUCLONE, $menuid, 0,
					"Создан новый сценарий '$menuname' (#{$jsonres['out_menuid']}) путем клонирования сценария #$menuid.");
			} else
				echo json_encode_safe($jsonresErr);
			break;
		case 'insert':
			$json = insertMenu($menuid, $menuid2);
			if (is_array($json) && isset($json['out_menu_id'])) {
				$jsonres = Array();
				$jsonres['ok'] = true;
				$jsonres['out_menuid'] = $json['out_menu_id'];
				//$logger->log('Insert menu:' . print_r(json_encode_safe($jsonres), true));
				echo json_encode_safe($jsonres);

				//changelog
				$menuname = $database_dynmenu->getMenuInfo($menuid);
				$menuname2 = $database_dynmenu->getMenuInfo($menuid2);
				$database_dynmenu->logMenuChange(CL_MENUS_MENUINSERT, $menuid2, 0,
					"Содержимое сценария '$menuname' (#$menuid) вставлено в сценарий '$menuname2' (#$menuid2).");
			} else
				echo json_encode_safe($jsonresErr);
			break;
		case 'blank':
			$json = newMenu($menuname);
			if (is_array($json) && isset($json['out_menu_id'])) {
				$jsonres = Array();
				$jsonres['ok'] = true;
				$jsonres['out_menuid'] = $json['out_menu_id'];
				//$logger->log('New blank menu:' . print_r(json_encode_safe($jsonres), true));
				echo json_encode_safe($jsonres);

				//changelog
				$database_dynmenu->logMenuChange(CL_MENUS_MENUBLANK, $menuid, 0,
					"Создан новый чистый сценарий '$menuname' (#{$jsonres['out_menuid']}).");
			} else
				echo json_encode_safe($jsonresErr);
			break;
		case 'delete':
			$json = deleteMenu($menuid);
			if (is_array($json) && isset($json['out_menu_id'])) {
				$jsonres = Array();
				$jsonres['ok'] = true;
				$jsonres['out_menuid'] = $json['out_menu_id'];
				//$logger->log('Delete menu:' . print_r(json_encode_safe($jsonres), true));
				echo json_encode_safe($jsonres);

				//changelog
				$database_dynmenu->logMenuChange(CL_MENUS_MENUDELETE, $menuid, 0,
					"Удален сценарий $menuid.");
			} else
				echo json_encode_safe($jsonresErr);
			break;
		case 'export':
			$menuname = urldecode($menuname);
			$logger->log("Export menu $menuid '$menuname'");

			if (empty($menuname))
				$menuname = $database_dynmenu->getMenuInfo($menuid);
			$exportfile = exportMenu($menuid, $menuname);

			header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
			header('Cache-Control: no-store, no-cache, must-revalidate');
			header('Cache-Control: post-check=0, pre-check=0', FALSE);
			header('Pragma: no-cache');
			header('Content-Disposition: attachment; filename="' . $menuname . '.txt";size=' . filesize($exportfile));
			header('Content-Transfer-Encoding: binary');
			header('Content-Length: ' . filesize($exportfile));
			header('Content-Description: File Transfer');

			if (($fp = fopen($exportfile, 'r'))) {
				while (!feof($fp)) {
					$buff = fread($fp, 4096);
					print $buff;
				}
				fclose($fp);
			}

			break;
		case 'import':
			$jsonres = array(
				'files' => $_FILES['menu_import_file']
			);
			if ($jsonres['files']['error'] !== UPLOAD_ERR_OK) {
				$jsonres['error'] = isset($upload_errors[$jsonres['files']['error']]) ? $upload_errors[$jsonres['files']['error']] : 'Неизвестная ошибка при загрузке файла';
			} elseif (($jsonres['files']['name'] == '') || ($jsonres['files']['size'] <= 0)) {
				$jsonres['error'] = 'Файл не задан или пуст';
			} elseif (($tmpfile = System::mktemp('menuimport_')) && move_uploaded_file($jsonres['files']['tmp_name'], $tmpfile)) {
				$logger->log("Import menu from file: {$jsonres['files']['name']}");

				$sqlcontents = file_get_contents($tmpfile);
				$sqlcontents = preg_replace('/\s*set term.*\n/i', '', $sqlcontents);
				if (substr($sqlcontents, 0, 3) == pack('CCC', 0xEF, 0xBB, 0xBF)) {//strip BOM if any
					$sqlcontents = substr($sqlcontents, 3);
					$logger->log("Import menu from file: Removed BOM from uploaded file!", PEAR_LOG_DEBUG);
				}

				$sqls = explode("^", $sqlcontents);
				//$sqls = preg_replace("/\s/", ' ', $sqls);

				$is_import_ok = true;

				$database_dynmenu->transactionStart();

				$database_dynmenu->transactionStart('tmp_table_cleanup');
				$database_dynmenu->query('delete from tmp_import');
				$database_dynmenu->transactionCommit('tmp_table_cleanup');

				$database_dynmenu->transactionStart('menu_import');
				foreach ($sqls as $q)
					if (trim($q) != '' && stripos($q, 'set term') === false && stripos($q, 'commit') === false) {
						$qres = $database_dynmenu->query($q);
						if (PEAR::isError($qres)) {
							$logger->log("Import menu from file ($q) ERR:" . getPEARErrorInfo($qres), PEAR_LOG_ERR);
							$is_import_ok = false;
							break;
						}
					}
				if ($is_import_ok) {
					$database_dynmenu->transactionCommit('menu_import');
					$database_dynmenu->transactionCommit();
					$logger->log("Import menu from file: {$jsonres['files']['name']} -> SUCCESS");

					$jsonres = array('ok' => true);
				} else {
					$database_dynmenu->transactionRollback('menu_import');
					$database_dynmenu->transactionRollback();
					$logger->log("Import menu from file: {$jsonres['files']['name']} -> FAILED");

					$jsonres = array('error' => 'При импорте возникли ошибки. Проверьте содержимое загруженного файла.');
				}
			} else {
				$jsonres['error'] = 'Внутренняя ошибка';
			}
			echo json_encode_safe($jsonres);
			break;
		default:
			echo json_encode_safe($jsonresErr);
	}
} else {
	echo json_encode_safe($jsonresErr);
}
?>
