set term ^;


/******************************************************************************/
/***                                 Tables                                 ***/
/******************************************************************************/


ALTER TABLE TB_DIALS_LIST ADD TMP_CON_DIALS VARCHAR(128);
^

CREATE TABLE TB_OFFICE_NUM (
    OF_NUMBER_ID      INTEGER NOT NULL,
    OF_ID             INTEGER,
    OF_NUMBER         FULL_NUMBER NOT NULL,
    OF_NUMBER_PREFIX  FULL_NUMBER,
    OF_DEFAULT_NUM    INTEGER
);
^

ALTER TABLE TB_OFFICE_USERS ADD
    USER_MAIL          VARCHAR(100);
^
ALTER TABLE TB_OFFICE_USERS ADD
    RECOVERY_PASSWORD  VARCHAR(128);
^


ALTER TABLE TB_OFFICES ADD OF_AUTOANSWER SMALLINT;
^

CREATE TABLE TB_USERACTION (
    ACT_ID       INTEGER NOT NULL,
    ACTGROUP_ID  INTEGER,
    ACT_NAME     VARCHAR(256),
    ACT_HINT     VARCHAR(256),
    "ACTION"     VARCHAR(1000),
    ACT_ORD      INTEGER,
    ACT_VISIBLE  SMALLINT,
    ACT_SYSTEM   SMALLINT
);
^

CREATE TABLE TB_USERACTION_GROUP (
    ACTGROUP_ID       INTEGER NOT NULL,
    ACTGROUP_NAME     VARCHAR(256),
    ACTGROUP_ORD      INTEGER,
    ACTGROUP_VISIBLE  SMALLINT
);
^

CREATE TABLE TB_USERMANUAL (
    USER_MANUAL_ID    INTEGER NOT NULL,
    USER_MANUAL_NAME  VARCHAR(256),
    USER_MANUAL_PATH  VARCHAR(256)
);
^

CREATE TABLE TB_USERRIGHT (
    ROLE_ID      INTEGER NOT NULL,
    ACT_ID       INTEGER NOT NULL,
    RIGHT_LEVEL  SMALLINT
);
^

ALTER TABLE TB_USERROLES ADD USER_MANUAL_ID INTEGER;
^

/******************************************************************************/
/***                              Primary Keys                              ***/
/******************************************************************************/
ALTER TABLE TB_OFFICE_NUM ADD CONSTRAINT PK_TB_OFFICE_NUM PRIMARY KEY (OF_NUMBER_ID);^
ALTER TABLE TB_USERACTION ADD CONSTRAINT PK_TB_USERACTION PRIMARY KEY (ACT_ID);^
ALTER TABLE TB_USERACTION_GROUP ADD CONSTRAINT PK_TB_USERACTION_GROUP PRIMARY KEY (ACTGROUP_ID);^
ALTER TABLE TB_USERMANUAL ADD CONSTRAINT PK_TB_USERMANUAL PRIMARY KEY (USER_MANUAL_ID);^
ALTER TABLE TB_USERRIGHT ADD CONSTRAINT PK_TB_USERRIGHT PRIMARY KEY (ROLE_ID, ACT_ID);^

/******************************************************************************/
/***                              Foreign Keys                              ***/
/******************************************************************************/
ALTER TABLE TB_OFFICE_NUM ADD CONSTRAINT FK_TB_OFFICE_NUM_1 FOREIGN KEY (OF_ID) REFERENCES TB_OFFICES (OF_ID);^
ALTER TABLE TB_USERACTION ADD CONSTRAINT FK_TB_USERACTION_1 FOREIGN KEY (ACTGROUP_ID) REFERENCES TB_USERACTION_GROUP (ACTGROUP_ID);^
ALTER TABLE TB_USERRIGHT ADD CONSTRAINT FK_TB_USERRIGHT_1 FOREIGN KEY (ROLE_ID) REFERENCES TB_USERROLES (ROLE_ID);^
ALTER TABLE TB_USERRIGHT ADD CONSTRAINT FK_TB_USERRIGHT_2 FOREIGN KEY (ACT_ID) REFERENCES TB_USERACTION (ACT_ID);^
ALTER TABLE TB_USERROLES ADD CONSTRAINT FK_TB_USERROLES_1 FOREIGN KEY (USER_MANUAL_ID) REFERENCES TB_USERMANUAL (USER_MANUAL_ID);^

/******************************************************************************/
/***                          Triggers for tables                           ***/
/******************************************************************************/

/* Trigger: TB_OFFICE_NUM_BI0 */
CREATE TRIGGER TB_OFFICE_NUM_BI0 FOR TB_OFFICE_NUM
ACTIVE BEFORE INSERT POSITION 0
AS
begin
  if ((new.OF_NUMBER_ID is null) or (new.OF_NUMBER_ID < 0)) then
  begin
    select max(OF_NUMBER_ID)+1
    from tb_office_num
    into new.OF_NUMBER_ID;

    new.OF_NUMBER_ID=coalesce(new.OF_NUMBER_ID,1);
  end
end
^

/* Trigger: TB_USERACTION_BD0 */
CREATE TRIGGER TB_USERACTION_BD0 FOR TB_USERACTION
ACTIVE BEFORE DELETE POSITION 0
AS
begin
    delete from tb_userright where right_level=0 and act_id in
  (select ACT_ID from tb_userright where ACT_ID=old.ACT_ID);
end
^

/* Trigger: TB_USERACTION_BI0 */
CREATE TRIGGER TB_USERACTION_BI0 FOR TB_USERACTION
ACTIVE BEFORE INSERT POSITION 0
AS
begin
  if ((new.ACT_ID is null) or (new.ACT_ID < 0)) then
  begin
    select max(ACT_ID)+1
    from tb_useraction
    into new.ACT_ID;

    new.ACT_ID=coalesce(new.ACT_ID,1);
  end
end
^

/* Trigger: TB_USERACTION_GROUP_BI0 */
CREATE TRIGGER TB_USERACTION_GROUP_BI0 FOR TB_USERACTION_GROUP
ACTIVE BEFORE INSERT POSITION 0
AS
begin
  if ((new.ACTGROUP_ID is null) or (new.ACTGROUP_ID < 0)) then
  begin
    select max(ACTGROUP_ID)+1
    from tb_useraction_group
    into new.ACTGROUP_ID;

    new.ACTGROUP_ID=coalesce(new.ACTGROUP_ID,1);
    /*if (new.ACTGROUP_ID is null) then
      new.ACTGROUP_ID = 1;*/
  end
end
^

/* Trigger: TB_USERROLES_BD0 */
CREATE TRIGGER TB_USERROLES_BD0 FOR TB_USERROLES
ACTIVE BEFORE DELETE POSITION 0
AS
begin
  delete from tb_userright where role_id=old.role_id;
end
^



/******************************************************************************/
/***                           Stored Procedures                            ***/
/******************************************************************************/

execute block
as
begin
    if(exists(select * from rdb$procedures where rdb$procedure_name='OF_MAKE_ROLEACTION')) then
    execute statement 'drop procedure OF_MAKE_ROLEACTION';
end
^
CREATE PROCEDURE OF_MAKE_ROLEACTION (
    USERROLE_ID INTEGER)
RETURNS (
    ACTGROUP_NAME VARCHAR(256),
    ACT_NAME VARCHAR(256),
    KEY1 SMALLINT,
    KEY2 SMALLINT,
    KEY3 SMALLINT,
    KEY4 SMALLINT,
    ROLE_ID INTEGER,
    ACT_ID INTEGER,
    ACT_VISIBLE INTEGER)
AS
BEGIN
  SUSPEND;
END
^

execute block
as
begin
    if(exists(select * from rdb$procedures where rdb$procedure_name='OF_SET_NUMBER')) then
    execute statement 'drop procedure OF_SET_NUMBER';
end
^
CREATE PROCEDURE OF_SET_NUMBER (
    OF_ID INTEGER,
    NEWNUMBER FULL_NUMBER,
    NEWNUMBER_PREFIX FULL_NUMBER,
    AUTOANSWER INTEGER)
RETURNS (
    OUT_RES INTEGER)
AS
BEGIN
  SUSPEND;
END
^

execute block
as
begin
    if(exists(select * from rdb$procedures where rdb$procedure_name='OF_UPDATE_USER')) then
    execute statement 'drop procedure OF_UPDATE_USER';
end
^
CREATE PROCEDURE OF_UPDATE_USER (
    USER_ID INTEGER,
    USER_NAME VARCHAR(256),
    USER_ROLE INTEGER,
    USER_PASSWORD VARCHAR(128),
    OF_ID INTEGER,
    USER_MAIL VARCHAR(100))
RETURNS (
    OUT_RES INTEGER)
AS
BEGIN
  SUSPEND;
END^

execute block
as
begin
    if(exists(select * from rdb$procedures where rdb$procedure_name='OF_WEB_AUTHORIZE')) then
    execute statement 'drop procedure OF_WEB_AUTHORIZE';
end
^
CREATE PROCEDURE OF_WEB_AUTHORIZE (
    OF_NUMBER VARCHAR(32),
    OF_USERNAME VARCHAR(256),
    OF_PASSWORD VARCHAR(128))
RETURNS (
    USER_ID INTEGER,
    USER_ROLE SMALLINT,
    USER_ROLE_NAME VARCHAR(256),
    OF_ID INTEGER,
    OF_DESCRIPTION VARCHAR(150),
    OF_MENU_ID INTEGER,
    USER_MANUAL_PATH VARCHAR(256),
    USER_MAIL VARCHAR(100))
AS
BEGIN
  SUSPEND;
END^







ALTER PROCEDURE OF_MAKE_ROLEACTION (
    USERROLE_ID INTEGER)
RETURNS (
    ACTGROUP_NAME VARCHAR(256),
    ACT_NAME VARCHAR(256),
    KEY1 SMALLINT,
    KEY2 SMALLINT,
    KEY3 SMALLINT,
    KEY4 SMALLINT,
    ROLE_ID INTEGER,
    ACT_ID INTEGER,
    ACT_VISIBLE INTEGER)
AS
begin

    begin
    insert into tb_userright (role_id, act_id, right_level)
    select coalesce(:USERROLE_ID,0) as role_id, A.act_id, 0 as right_level  from
    tb_useraction A left join
    (select role_id, act_id from tb_userright where role_id=:USERROLE_ID) as R
    on R.act_id=A.act_id where R.role_id is null;
    end

for
select
G.actgroup_name,
A.act_name,
bin_and(8,R.right_level)/8 as key1,
bin_and(4,R.right_level)/4 as key2,
bin_and(2,R.right_level)/2 as key3,
bin_and(1,R.right_level)/1 as key4,
R.role_id,
R.act_id,
coalesce(A.act_visible,0)
from
tb_userright R
inner join tb_useraction A on R.act_id=A.act_id
inner join tb_useraction_group G on A.actgroup_id=G.actgroup_id
where R.role_id=:USERROLE_ID
order by G.actgroup_ord,A.act_ord
into
    :actgroup_name,
    :act_name,
    :key1,
    :key2,
    :key3,
    :key4,
    :role_id,
    :act_id,
    :act_visible
    do
    suspend;


end
^



ALTER PROCEDURE OF_SET_NUMBER (
    OF_ID INTEGER,
    NEWNUMBER FULL_NUMBER,
    NEWNUMBER_PREFIX FULL_NUMBER,
    AUTOANSWER INTEGER)
RETURNS (
    OUT_RES INTEGER)
AS
begin
  /*update prefix*/
  update tb_offices
  set
  of_number_prefix = :newnumber_prefix,
  of_autoanswer = :autoanswer
  where of_id = :of_id;

  /*try update number*/
  select -of_id
  from tb_offices
  where of_id <> :of_id and
        of_number = :newnumber
  into out_res;
  if (out_res is null) then
  begin
    update tb_offices
    set of_number = :newnumber
    where of_id = :of_id;

    out_res = :of_id;
  end

  suspend;
end
^

ALTER PROCEDURE OF_UPDATE_USER (
    USER_ID INTEGER,
    USER_NAME VARCHAR(256),
    USER_ROLE INTEGER,
    USER_PASSWORD VARCHAR(128),
    OF_ID INTEGER,
    USER_MAIL VARCHAR(100))
RETURNS (
    OUT_RES INTEGER)
AS
declare variable TMP_ID integer;
begin
  if (of_id is not null and
      not exists(select 1
                 from tb_offices
                 where of_id = :of_id)) then
  begin
    out_res = null;
  end
  else
  begin
    if (user_id is null) then
    begin
      /*add user*/
      out_res = 1;
      tmp_id = 1;
      while (tmp_id is not null) do
      begin
        tmp_id = null;
        select user_id
        from tb_office_users
        where user_id = :out_res
        into :tmp_id;
        if (tmp_id is not null) then
          out_res = out_res + 1;
      end

      insert into tb_office_users (user_id, user_name, user_password, user_role, of_id, add_date, user_mail)
      values (:out_res, :user_name, :user_password, :user_role, :of_id, 'Now', :user_mail);
    end
    else
    if (user_id < 0) then
    begin
      /*delete user*/
      if (of_id is null) then
      begin
        /*block deletion of last system admin*/
        select count(user_id)
        from tb_office_users
        where of_id is null
        into :out_res;

        if (out_res < 2) then
          out_res = null;
        else
        begin
          delete
          from tb_office_users
          where user_id = -:user_id;

          out_res = -:user_id;
        end
      end
      else
      begin
        delete
        from tb_office_users
        where user_id = -:user_id;

        out_res = -:user_id;
      end
    end
    else
    begin
      /*modify user*/
      if (user_password is null) then
        update tb_office_users
        set user_name = :user_name,
            user_role = :user_role,
            user_mail = :user_mail
        where user_id = :user_id;
      else
        update tb_office_users
        set user_name = :user_name,
            user_password = :user_password,
            user_role = :user_role,
            user_mail = :user_mail
        where user_id = :user_id;

      out_res = :user_id;
    end

    suspend;
  end
end
^


ALTER PROCEDURE OF_WEB_AUTHORIZE (
    OF_NUMBER VARCHAR(32),
    OF_USERNAME VARCHAR(256),
    OF_PASSWORD VARCHAR(128))
RETURNS (
    USER_ID INTEGER,
    USER_ROLE SMALLINT,
    USER_ROLE_NAME VARCHAR(256),
    OF_ID INTEGER,
    OF_DESCRIPTION VARCHAR(150),
    OF_MENU_ID INTEGER,
    USER_MANUAL_PATH VARCHAR(256),
    USER_MAIL VARCHAR(100))
AS
begin
  if (of_number is not null and
      of_username is not null) then
  begin
    select u.user_id,
           u.user_role,
           r.role_name,
           u.of_id,
           o.of_describer,
           o.of_menu_id,
           m.user_manual_path,
           u.user_mail
    from tb_office_users u
    inner join tb_offices o on (o.of_id = u.of_id)
    inner join tb_userroles r on (r.role_id = u.user_role)
    left join tb_usermanual m on r.user_manual_id=m.user_manual_id
    where (u.user_name = :of_username) and
          (:of_password is null or (u.user_password = :of_password)) and
          (o.of_number = :of_number)
    into :user_id,
         :user_role,
         :user_role_name,
         :of_id,
         :of_description,
         :of_menu_id,
         :user_manual_path,
         :user_mail;
  end
  else
  if (of_number is null) then
  begin
    /* admin */
    select u.user_id,
           u.user_role,
           r.role_name,
           m.user_manual_path,
           u.user_mail
    from tb_office_users u
    inner join tb_userroles r on (r.role_id = u.user_role)
    left join tb_usermanual m on r.user_manual_id=m.user_manual_id
    where (u.of_id is null) and
          (u.user_name = :of_username) and
          (:of_password is null or (u.user_password = :of_password))
    into :user_id,
         :user_role,
         :user_role_name,
         :user_manual_path,
         :user_mail;
  end
  suspend;
end
^

ALTER PROCEDURE OFFICE_SELECT (
    CALLEDID VARCHAR(64))
RETURNS (
    RESULT VARCHAR(32),
    OFFICE__ID INTEGER,
    OFFICE__MENUID INTEGER)
AS
declare variable OFID integer;
begin
  /**
   *      
   */
  result = '';
  office__id = -1;
  office__menuid = -1;

  select of_id,
         of_menu_id 
  from tb_offices 
  where (of_number = :calledid) 
  into :ofid,
       :office__menuid;

  if (ofid is null) then
  begin
  select F.of_id,
         F.of_menu_id
  from tb_offices F inner join tb_office_num N on F.of_id = N.of_id
  where (N.of_number = :calledid)
  into :ofid,
       :office__menuid;
  end

  if (ofid is not null) then
  begin
    result = 'OK';
    office__id = ofid;
  end 

  suspend;
end
^


/******************************************************************************/
/***                                                              ***/
/******************************************************************************/

execute block
as
    declare of_id integer;
    declare of_number varchar(32);
    declare of_number_prefix varchar(32);
    declare of_defaul_num integer;
begin
    for
    select OF_ID,OF_NUMBER,OF_NUMBER_PREFIX,1 as OF_DEFAULT_NUM from tb_offices
    into :of_id, :of_number, :of_number_prefix, :of_defaul_num
    do
    begin
        insert into tb_office_num (OF_ID,OF_NUMBER,OF_NUMBER_PREFIX,OF_DEFAULT_NUM)
        values (:of_id,:of_number,:of_number_prefix,:of_defaul_num);
    end
end
^



execute block
as
begin
INSERT INTO TB_USERACTION_GROUP (ACTGROUP_ID, ACTGROUP_NAME, ACTGROUP_ORD, ACTGROUP_VISIBLE) VALUES (0, '< >', 0, 1);
INSERT INTO TB_USERACTION_GROUP (ACTGROUP_ID, ACTGROUP_NAME, ACTGROUP_ORD, ACTGROUP_VISIBLE) VALUES (1, ' ', 1, 0);
INSERT INTO TB_USERACTION_GROUP (ACTGROUP_ID, ACTGROUP_NAME, ACTGROUP_ORD, ACTGROUP_VISIBLE) VALUES (2, ' ', 2, 0);
INSERT INTO TB_USERACTION_GROUP (ACTGROUP_ID, ACTGROUP_NAME, ACTGROUP_ORD, ACTGROUP_VISIBLE) VALUES (3, ' ', 3, NULL);
INSERT INTO TB_USERACTION_GROUP (ACTGROUP_ID, ACTGROUP_NAME, ACTGROUP_ORD, ACTGROUP_VISIBLE) VALUES (4, '', 4, NULL);
INSERT INTO TB_USERACTION_GROUP (ACTGROUP_ID, ACTGROUP_NAME, ACTGROUP_ORD, ACTGROUP_VISIBLE) VALUES (5, '', 5, 0);
INSERT INTO TB_USERACTION_GROUP (ACTGROUP_ID, ACTGROUP_NAME, ACTGROUP_ORD, ACTGROUP_VISIBLE) VALUES (6, ' ', 6, 0);
INSERT INTO TB_USERACTION_GROUP (ACTGROUP_ID, ACTGROUP_NAME, ACTGROUP_ORD, ACTGROUP_VISIBLE) VALUES (7, 'SMS-', 7, 0);
end
^

execute block
as
begin
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (1, 1, ' ', '  ', 'right_group.php', 1, NULL, 1);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (2, 1, ' ', '  ', 'right_action.php', 2, NULL, 1);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (3, 1, ' ', '  ', 'right_role.php', 3, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (9, 2, ' ', '  ', 'daygroups.php', 1, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (10, 2, ' ', '  ', 'graphs.php', 2, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (11, 2, ' ', '   ', 'numbers_ext.php', 3, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (12, 2, ' ', '  ', 'dials.php', 4, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (13, 2, ' ', '   ', 'numbers_int.php', 5, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (14, 1, '', '  ', 'right_user.php', 4, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (44, 3, ' ', '    ', 'synths.php', 2, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (46, 4, ' ', '  ', 'stats_fax.php', 2, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (47, 4, ' ', '  ', 'stats_voice.php', 1, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (48, 0, ' ', '  ', 'main.php', 1, 0, 0);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (49, 3, '', ' ', 'setup.php', 3, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (50, 1, ' (old)', '   ', 'adm_users.php', 5, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (51, 1, '', '  ', 'adm_offices.php', 6, 0, 0);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (53, 0, ' ', '  ', 'main.php', 2, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (54, 3, '   ', '     ', 'numbers_bw.php', 4, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (55, 5, '', '   ', 'mb.php', 1, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (56, 5, '', '   ', 'stats.php', 2, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (57, 5, ' ', '    ', 'changelog.php', 3, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (58, 3, ' ', '   ', 'adm_menuedit.php', 1, NULL, NULL);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (59, 1, ' (/ )', '  ', 'adm_offices_make', 7, 1, 0);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (60, 6, ' ', '  ', 'acnt_plan.php', 1, 0, 0);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (61, 6, 'SIP ', ' SIP ', 'acnt_sip.php', 2, 0, 0);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (62, 7, ' ', '  SMS', 'sms_send.php', 1, 0, 0);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (63, 7, ' ', '   SMS', 'sms_report.php', 2, 0, 0);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (64, 6, '', '  ', 'techsupport.php', 3, 0, 1);
INSERT INTO TB_USERACTION (ACT_ID, ACTGROUP_ID, ACT_NAME, ACT_HINT, "ACTION", ACT_ORD, ACT_VISIBLE, ACT_SYSTEM) VALUES (65, 1, ' ', ' ', 'chatsupport.php', 8, 0, 1);
end
^

execute block
as
begin
INSERT INTO TB_USERMANUAL (USER_MANUAL_ID, USER_MANUAL_NAME, USER_MANUAL_PATH) VALUES (0, ' ', NULL);
INSERT INTO TB_USERMANUAL (USER_MANUAL_ID, USER_MANUAL_NAME, USER_MANUAL_PATH) VALUES (1, ' ', 'docs/VirtualOfficeAdminManual_rino.pdf');
INSERT INTO TB_USERMANUAL (USER_MANUAL_ID, USER_MANUAL_NAME, USER_MANUAL_PATH) VALUES (2, ' ', 'docs/VirtualOfficeOwnerManual_rino.pdf');
INSERT INTO TB_USERMANUAL (USER_MANUAL_ID, USER_MANUAL_NAME, USER_MANUAL_PATH) VALUES (3, ' ', 'docs/VirtualOfficeLocalAdminManual_rino.pdf');
end
^

execute block
as
begin
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 1, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 2, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 3, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 9, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 10, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 11, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 12, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 13, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 14, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 44, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 46, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 47, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 48, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 49, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 50, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 51, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 53, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 54, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 55, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 56, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 57, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 58, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 59, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 60, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 61, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 62, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 63, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 64, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (0, 65, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 1, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 2, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 3, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 9, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 10, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 11, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 12, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 13, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 14, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 44, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 46, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 47, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 48, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 49, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 50, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 51, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 53, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 54, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 55, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 56, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 57, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 58, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 59, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 60, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 61, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 62, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 63, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 64, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (1, 65, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 1, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 2, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 3, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 9, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 10, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 11, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 12, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 13, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 14, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 44, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 46, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 47, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 48, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 49, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 50, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 51, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 53, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 54, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 55, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 56, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 57, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 58, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 59, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 60, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 61, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 62, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 63, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 64, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (2, 65, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 1, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 2, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 3, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 9, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 10, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 11, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 12, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 13, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 14, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 44, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 46, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 47, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 48, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 49, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 50, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 51, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 53, 4);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 54, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 55, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 56, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 57, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 58, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 59, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 60, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 61, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 62, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 63, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 64, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (10, 65, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 1, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 2, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 3, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 9, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 10, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 11, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 12, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 13, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 14, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 44, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 46, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 47, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 48, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 49, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 50, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 51, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 53, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 54, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 55, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 56, 8);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 57, 12);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 58, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 59, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 60, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 61, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 62, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 63, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 64, 0);
INSERT INTO TB_USERRIGHT (ROLE_ID, ACT_ID, RIGHT_LEVEL) VALUES (200, 65, 0);
end
^



commit^







set term ;^

